/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GuiTools.cpp,v 2.6 1999/10/16 01:43:44 jeff Exp $

*******************************************************************************/


#include <qapplication.h>
#include <qcdestyle.h>
#include <qmotifstyle.h>
#include <qplatinumstyle.h>
#include <qstrlist.h>
#include <qwindowsstyle.h>

#include "GuiTools.h"


QStrList GuiTools::itsQDialogSlots;


const QString GuiTools::platinumStyle = "Platinum";
const QString GuiTools::cdeStyle = "CDE";
const QString GuiTools::motifStyle = "Motif";
const QString GuiTools::windowsStyle = "Windows";


void GuiTools::SetStyle( const QString& style )
{
    if( style == platinumStyle )
    {
        QApplication::setStyle( new QPlatinumStyle() );
    }
    else if( style == cdeStyle )
    {
        QApplication::setStyle( new QCDEStyle( TRUE ) );
    }
    else if( style == motifStyle )
    {
        QApplication::setStyle( new QMotifStyle( TRUE ) );
    }
    else if( style == windowsStyle )
    {
        QApplication::setStyle( new QWindowsStyle() );
    }
}


void GuiTools::SetVisible( QWidget*	widget,
			   bool		visible )
{
    if( visible )
    {
	widget->show();
    }
    else
    {
	widget->hide();
    }
}


QRect GuiTools::GetGlobalGeometry
(
    const QWidget*	widget
)
{
    //const QRect& rect = widget->geometry();

//     return QRect( widget->mapToGlobal( rect.topLeft() ),
// 		  widget->mapToGlobal( rect.bottomRight() ) );

    return QRect( widget->mapToGlobal( QPoint(0,0) ),
		  widget->size() );
}


void GuiTools::ChangeColor( 
    QWidget*            widget,
    const QColor&       color 
    )
{
    widget->setPalette( QPalette( color ) );
}


QString GuiTools::QWidgetFocusPolicyStr(
    QWidget::FocusPolicy	theFocusPolicy
    )
{
    QString str;
    
    switch( theFocusPolicy )
    {
	case QWidget::TabFocus:
	{
	    str = "TabFocus";
	    break;
	}
	case QWidget::ClickFocus:
	{
	    str = "ClickFocus";
	    break;
	}
	case QWidget::StrongFocus:
	{
	    str = "StrongFocus";
	    break;
	}
	case QWidget::WheelFocus:
	{
	    str = "WheelFocus";
	    break;
	}
	case QWidget::NoFocus:
	{
	    str = "NoFocus";
	    break;
	}
    }
    
    return str;
}


QWidget::FocusPolicy GuiTools::QWidgetFocusPolicy(
    const QString&	theFocusPolicyStr
    )
{
    QWidget::FocusPolicy focusPolicy = QWidget::NoFocus;
    
    if( theFocusPolicyStr == "TabFocus" )
    {
	focusPolicy = QWidget::TabFocus;
    }
    else if( theFocusPolicyStr == "ClickFocus" )
    {
	focusPolicy = QWidget::ClickFocus;
    }
    else if( theFocusPolicyStr == "StrongFocus" )
    {
	focusPolicy = QWidget::StrongFocus;
    }
    else if( theFocusPolicyStr == "WheelFocus" )
    {
	focusPolicy = QWidget::WheelFocus;
    }
    else if( theFocusPolicyStr == "NoFocus" )
    {
	focusPolicy = QWidget::NoFocus;
    }
    
    return focusPolicy;
}


QString GuiTools::QWidgetBackgroundModeStr(
    QWidget::BackgroundMode	theBackgroundMode
    )
{
    QString str;
    
    switch( theBackgroundMode )
    {
	case QWidget::FixedColor:
	{
	    str = "FixedColor";
	    break;
	}
	case QWidget::FixedPixmap:
	{
	    str = "FixedPixmap";
	    break;
	}
	case QWidget::NoBackground:
	{
	    str = "NoBackground";
	    break;
	}
	case QWidget::PaletteForeground:
	{
	    str = "PaletteForeground";
	    break;
	}
	case QWidget::PaletteBackground:
	{
	    str = "PaletteBackground";
	    break;
	}
	case QWidget::PaletteLight:
	{
	    str = "PaletteLight";
	    break;
	}
	case QWidget::PaletteMidlight:
	{
	    str = "PaletteMidlight";
	    break;
	}
	case QWidget::PaletteDark:
	{
	    str = "PaletteDark";
	    break;
	}
	case QWidget::PaletteMid:
	{
	    str = "PaletteMid";
	    break;
	}
	case QWidget::PaletteText:
	{
	    str = "PaletteText";
	    break;
	}
	case QWidget::PaletteBase:
	{
	    str = "PaletteBase";
	    break;
	}
        case QWidget::PaletteButton:
        {
            str = "PaletteButton";
            break;
        }
        case QWidget::PaletteBrightText:
        {
            str = "PaletteBrightText";
            break;
        }
        case QWidget::PaletteShadow:
        {
            str = "PaletteShadow";
            break;
        }
        case QWidget::PaletteHighlight:
        {
            str = "PaletteHighlight";
            break;
        }
        case QWidget::PaletteHighlightedText:
        {
            str = "PaletteHighlightedText";
            break;
        }
	default:
	{
            debug( "Background mode: %d", theBackgroundMode );
            break;
	}
    }
    
    return str;
}
	

QWidget::BackgroundMode GuiTools::QWidgetBackgroundMode(
    const QString&	theBackgroundModeStr
    )
{
    QWidget::BackgroundMode backgroundMode = QWidget::PaletteBackground;
    
    if( theBackgroundModeStr == "FixedColor" )
    {
	backgroundMode = QWidget::FixedColor;
    }
    else if( theBackgroundModeStr == "FixedPixmap" )
    {
	backgroundMode = QWidget::FixedPixmap;
    }
    else if( theBackgroundModeStr == "NoBackground" )
    {
	backgroundMode = QWidget::NoBackground;
    }
    else if( theBackgroundModeStr == "PaletteForeground" )
    {
	backgroundMode = QWidget::PaletteForeground;
    }
    else if( theBackgroundModeStr == "PaletteBackground" )
    {
	backgroundMode = QWidget::PaletteBackground;
    }
    else if( theBackgroundModeStr == "PaletteLight" )
    {
	backgroundMode = QWidget::PaletteLight;
    }
    else if( theBackgroundModeStr == "PaletteMidlight" )
    {
	backgroundMode = QWidget::PaletteMidlight;
    }
    else if( theBackgroundModeStr == "PaletteDark" )
    {
	backgroundMode = QWidget::PaletteDark;
    }
    else if( theBackgroundModeStr == "PaletteMid" )
    {
	backgroundMode = QWidget::PaletteMid;
    }
    else if( theBackgroundModeStr == "PaletteText" )
    {
	backgroundMode = QWidget::PaletteText;
    }
    else if( theBackgroundModeStr == "PaletteBase" )
    {
	backgroundMode = QWidget::PaletteBase;
    }
    else if( theBackgroundModeStr == "PaletteButton" )
    {
        backgroundMode = QWidget::PaletteButton;
    }
    else if( theBackgroundModeStr == "PaletteBrightText" )
    {
        backgroundMode = QWidget::PaletteBrightText;
    }
    else if( theBackgroundModeStr == "PaletteShadow" )
    {
        backgroundMode = QWidget::PaletteShadow;
    }
    else if( theBackgroundModeStr == "PaletteHighlight" )
    {
        backgroundMode = QWidget::PaletteHighlight;
    }
    else if( theBackgroundModeStr == "PaletteHighlightedText" )
    {
        backgroundMode = QWidget::PaletteHighlightedText;
    }

    return backgroundMode;
}


QString GuiTools::QWidgetPropagationModeStr(
    QWidget::PropagationMode	thePropagationMode
    )
{
    QString str;
    
    switch( thePropagationMode )
    {
	case QWidget::NoChildren:
	{
	    str = "NoChildren";
	    break;
	}
	case QWidget::SameFont:
	    //case QWidget::SamePalette:
	{
	    str = "SameFont";
	    break;
	}
	case QWidget::AllChildren:
	{
	    str = "AllChildren";
	    break;
	}
    }
    
    return str;
}


QWidget::PropagationMode GuiTools::QWidgetPropagationMode(
    const QString&	thePropagationModeStr
    )
{
    QWidget::PropagationMode propMode = QWidget::NoChildren;
    
    if( thePropagationModeStr == "NoChildren" )
    {
	propMode = QWidget::NoChildren;
    }
    else if( thePropagationModeStr == "SameFont" )
    {
	propMode = QWidget::SameFont;
    }
    else if( thePropagationModeStr == "SamePalette" )
    {
	propMode = QWidget::SamePalette;
    }
    else if( thePropagationModeStr == "AllChildren" )
    {
	propMode = QWidget::AllChildren;
    }
    
    return propMode;
}


QString GuiTools::AlignmentStr(
    int	theAlignment
    )
{
    QStrList strList;

    if( theAlignment & Qt::AlignLeft )
    {
	strList.append( "AlignLeft" );
    }
    else if( theAlignment & Qt::AlignHCenter )
    {
	strList.append( "AlignHCenter" );
    }
    else if( theAlignment & Qt::AlignRight )
    {
	strList.append( "AlignRight" );
    }
    
    if( theAlignment & Qt::AlignTop )
    {
	strList.append( "AlignTop" );
    }
    else if( theAlignment & Qt::AlignVCenter )
    {
	strList.append( "AlignVCenter" );
    }
    else if( theAlignment & Qt::AlignBottom )
    {
	strList.append( "AlignBottom" );
    }

    if( theAlignment & Qt::SingleLine )
    {
        strList.append( "SingleLine" );
    }
    
    if( theAlignment & Qt::DontClip )
    {
        strList.append( "DontClip" );
    }
    
    if( theAlignment & Qt::ExpandTabs )
    {
        strList.append( "ExpandTabs" );
    }

    if( theAlignment & Qt::ShowPrefix )
    {
        strList.append( "ShowPrefix" );
    }
    
    if( theAlignment & Qt::WordBreak )
    {
        strList.append( "WordBreak" );
    }
    
    if( theAlignment & Qt::DontPrint )
    {
        strList.append( "DontPrint" );
    }
    
    QString alignStr;
    QStrListIterator iter( strList );
    char* str = 0;
    while( (str = iter()) )
    {
        if( !alignStr.isEmpty() )
        {
            alignStr += "|";
        }
        
        alignStr += str;
    }
    
    return alignStr;
}


int GuiTools::Alignment(
    const QString&	theAlignmentStr
    )
{
    int align = 0;
    QString str = theAlignmentStr;
    QString alignStr;
    while( !str.isEmpty() )
    {
        int pos = str.find( '|' );
        if( pos > -1 )
        {
            alignStr = str.left( pos ).stripWhiteSpace();
            str = str.mid( pos + 1, str.length() - pos - 1 ).stripWhiteSpace();
        }
        else
        {
            alignStr = str;
            str = "";
        }
	
        if( alignStr == "AlignLeft" )
        {
            align |= Qt::AlignLeft;
        }
        else if( alignStr == "AlignHCenter" )
        {
            align |= Qt::AlignHCenter;
        }
        else if( alignStr == "AlignRight" )
        {
            align |= Qt::AlignRight;
        }
        else if( alignStr == "AlignTop" )
        {
            align |= Qt::AlignTop;
        }
        else if( alignStr == "AlignVCenter" )
        {
	    align |= Qt::AlignVCenter;
        }
        else if( alignStr == "AlignBottom" )
        {
            align |= Qt::AlignBottom;
        }
        else if( alignStr == "SingleLine" )
        {
            align |= Qt::SingleLine;
        }
        else if( alignStr == "DontClip" )
        {
            align |= Qt::DontClip;
        }
        else if( alignStr == "ExpandTabs" )
        {
            align |= Qt::ExpandTabs;
        }
        else if( alignStr == "ShowPrefix" )
        {
            align |= Qt::ShowPrefix;
        }
        else if( alignStr == "WordBreak" )
        {
            align |= Qt::WordBreak;
        }
        else if( alignStr == "DontPrint" )
        {
            align |= Qt::DontPrint;
        }
    }

    return align;
}


void GuiTools::TransferListBox( const QListBox*	source,
				QListBox*	dest )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}    


void GuiTools::TransferListFromCombo( const QComboBox*	source,
				      QListBox*		dest )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}


void GuiTools::TransferListToCombo( QComboBox*		dest,
				    const QListBox*	source )
{
    dest->clear();
    for( unsigned int i = 0; i < (unsigned int)source->count(); i++ )
        dest->insertItem( source->text( i ) );
}


QString GuiTools::QFontStyleHintStr( QFont::StyleHint	hint )
{
    QString str;
    
    switch( hint )
    {
	case QFont::Helvetica:
	default:
	{
	    str = "Helvetica";
	    break;
	}
	case QFont::Times:
	{
	    str = "Times";
	    break;
	}
	case QFont::Courier:
	{
	    str = "Courier";
	    break;
	}
	case QFont::OldEnglish:
	{
	    str = "Old English";
	    break;
	}
	case QFont::System:
	{
	    str = "System";
	    break;
	}
	case QFont::AnyStyle:
	{
	    str = "AnyStyle";
	    break;
	}
    }

    return str;
}


QFont::StyleHint GuiTools::QFontStyleHint( QString	str )
{
    QFont::StyleHint hint = QFont::Helvetica;
    
    if( (str == "Helvetica") || (str == "Sans Serif") )
    {
	// already set to helvetica
    }
    else if( (str == "Times") || (str == "Serif") )
    {
	hint = QFont::Times;
    }
    else if( (str == "Courier") || (str == "Type Writer") )
    {
	hint = QFont::Courier;
    }
    else if( (str == "Old English") || (str == "Decorative") )
    {
	hint = QFont::OldEnglish;
    }
    else if( str == "System" )
    {
	hint = QFont::System;
    }
    else if( str == "Any Style" )
    {
	hint = QFont::AnyStyle;
    }

    return hint;
}



QString GuiTools::QFontWeightStr( int	weight )
{
    QString str;
    str.setNum( weight );
    
    switch( weight )
    {
	case QFont::Light:
	{
	    str = "Light";
	    break;
	}
	case QFont::Normal:
	{
	    str = "Normal";
	    break;
	}
	case QFont::DemiBold:
	{
	    str = "DemiBold";
	    break;
	}
	case QFont::Bold:
	{
	    str = "Bold";
	    break;
	}
	case QFont::Black:
	{
	    str = "Black";
	    break;
	}
    }

    return str;
}


QString GuiTools::QFontCharSetStr( QFont::CharSet	set )
{
    QString str;
    
    switch( set )
    {
	case QFont::Latin1:
	{
	    str = "Latin1";
	    break;
	}
	case QFont::AnyCharSet:
	{
	    str = "Any Char Set";
	    break;
	}
	case QFont::Latin2:
	{
	    str = "Latin2";
	    break;
	}
	case QFont::Latin3:
	{
	    str = "Latin3";
	    break;
	}
	case QFont::Latin4:
	{
	    str = "Latin4";
	    break;
	}
	case QFont::Latin6:
	{
	    str = "Latin6";
	    break;
	}
	case QFont::Latin7:
	{
	    str = "Latin7";
	    break;
	}
	case QFont::Latin8:
	{
	    str = "Latin8";
	    break;
	}
	case QFont::Latin9:
	{
	    str = "Latin9";
	    break;
	}
	case QFont::ISO_8859_5:
	{
	    str = "ISO_8859_5";
	    break;
	}
	case QFont::ISO_8859_6:
	{
	    str = "ISO_8859_6";
	    break;
	}
	case QFont::ISO_8859_7:
	{
	    str = "ISO_8859_7";
	    break;
	}
	case QFont::ISO_8859_8:
	{
	    str = "ISO_8859_8";
	    break;
	}
	case QFont::ISO_8859_9:
	{
	    str = "ISO_8859_9";
	    break;
	}
	case QFont::ISO_8859_11:
	{
	    str = "ISO_8859_11";
	    break;
	}
	case QFont::ISO_8859_12:
	{
	    str = "ISO_8859_12";
	    break;
	}
	case QFont::KOI8R:
	{
	    str = "KOI8R";
	    break;
	}
	case QFont::Set_Ja:
	{
	    str = "Set_Ja";
	    break;
	}
	case QFont::Set_Ko:
	{
	    str = "Set_Ko";
	    break;
	}
	case QFont::Set_Th_TH:
	{
	    str = "Set_Th_TH";
	    break;
	}
	case QFont::Set_Zh:
	{
	    str = "Set_Zh";
	    break;
	}
	case QFont::Set_Zh_TW:
	{
	    str = "Set_Zh_TW";
	    break;
	}
	case QFont::Unicode:
	{
	    str = "Unicode";
	    break;
	}
    }
    
    return str;
}


QFont::CharSet GuiTools::QFontCharSet( QString		set )
{
    QFont::CharSet charSet = QFont::Latin1;
    
    if( (set == "Latin1") || (set == "ISO_8859_1") )
    {
	// already set to Latin1
    }
    else if( set == "Any Char Set" )
    {
	charSet = QFont::AnyCharSet;
    }
    else if( (set == "Latin2") || (set == "ISO_8859_2") )
    {
	charSet = QFont::Latin2;
    }
    else if( (set == "Latin3") || (set == "ISO_8859_3") )
    {
	charSet = QFont::Latin3;
    }
    else if( (set == "Latin4") || (set == "ISO_8859_4") )
    {
	charSet = QFont::Latin4;
    }
    else if( (set == "Latin5") || (set == "ISO_8859_5") )
    {
	charSet = QFont::ISO_8859_5;
    }
    else if( (set == "Latin6") || (set == "ISO_8859_6") )
    {
	charSet = QFont::ISO_8859_6;
    }
    else if( (set == "Latin7") || (set == "ISO_8859_7") )
    {
	charSet = QFont::ISO_8859_7;
    }
    else if( (set == "Latin8") || (set == "ISO_8859_8") )
    {
	charSet = QFont::ISO_8859_8;
    }
    else if( (set == "Latin9") || (set == "ISO_8859_9") )
    {
	charSet = QFont::ISO_8859_9;
    }
    else if( set == "ISO_8859_10" )
    {
	charSet = QFont::ISO_8859_10;
    }
    else if( set == "ISO_8859_11" )
    {
	charSet = QFont::ISO_8859_11;
    }
    else if( set == "ISO_8859_12" )
    {
	charSet = QFont::ISO_8859_12;
    }
    else if( set == "ISO_8859_13" )
    {
	charSet = QFont::ISO_8859_13;
    }
    else if( set == "ISO_8859_14" )
    {
	charSet = QFont::ISO_8859_14;
    }
    else if( set == "ISO_8859_15" )
    {
	charSet = QFont::ISO_8859_15;
    }
    else if( set == "KOI8R" )
    {
	charSet = QFont::KOI8R;
    }
    else if( (set == "Set_Ja") || (set == "Set_1") )
    {
	charSet = QFont::Set_Ja;
    }
    else if( set == "Set_Ko" )
    {
	charSet = QFont::Set_Ko;
    }
    else if( set == "Set_Th_TH" )
    {
	charSet = QFont::Set_Th_TH;
    }
    else if( set == "Set_Zh" )
    {
	charSet = QFont::Set_Zh;
    }
    else if( (set == "Set_Zh_TW") || (set == "Set_N") )
    {
	charSet = QFont::Set_Zh_TW;
    }
    else if( set == "Unicode" )
    {
	charSet = QFont::Unicode;
    }
    
    return charSet;
}


void GuiTools::QStrListAppendAll( QStrList&		to,
				  const QStrList&	from )
{
    QStrListIterator iter( from );
    char* item;
    
    while( (item = iter()) )
    {
	to.append( item );
    }
}


QString GuiTools::QLCDNumberSegStyleStr( QLCDNumber::SegmentStyle	style )
{
    QString str = "Outline";
    
    switch( style )
    {
	case QLCDNumber::Outline:
	default:
	{
	    // already set to Outline
	    break;
	}
	case QLCDNumber::Filled:
	{
	    str = "Filled";
	    break;
	}
	case QLCDNumber::Flat:
	{
	    str = "Flat";
	    break;
	}
    }
    
    return str;
}


QLCDNumber::SegmentStyle GuiTools::QLCDNumberSegStyle( QString		str )
{
    QLCDNumber::SegmentStyle style = QLCDNumber::Outline;
    
    if( str == "Filled" )
    {
	style = QLCDNumber::Filled;
    }
    else if( str == "Flat" )
    {
	style = QLCDNumber::Flat;
    }
    
    return style;
}


QString GuiTools::QLineEditEchoModeStr( QLineEdit::EchoMode	mode )
{
    QString str= "Normal";
    
    switch( mode )
    {
	case QLineEdit::Normal:
	default:
	{
	    // already set to Normal
	    break;
	}
	case QLineEdit::NoEcho:
	{
	    str = "NoEcho";
	    break;
	}
	case QLineEdit::Password:
	{
	    str = "Password";
	    break;
	}
    }
    
    return str;
}


QLineEdit::EchoMode GuiTools::QLineEditEchoMode( QString	str )
{
    QLineEdit::EchoMode echo = QLineEdit::Normal;
    
    if( str == "NoEcho" )
    {
	echo = QLineEdit::NoEcho;
    }
    else if( str == "Password" )
    {
	echo = QLineEdit::Password;
    }
    
    return echo;
}


QString GuiTools::QSliderTickSettingStr( QSlider::Orientation	orient,
					 QSlider::TickSetting	tick )
{
    QString str = "NoMarks";
    

    if( orient == QSlider::Vertical )
    {
	if( tick == QSlider::Left )
	{
	    str = "Left";
	}
	else if( tick == QSlider::Right )
	{
	    str = "Right";
	}
    }
    else if( orient == QSlider::Horizontal )
    {
	if( tick == QSlider::Above )
	{
	    str = "Above";
	}
	else if( tick == QSlider::Below )
	{
	    str = "Below";
	}
    }

    if( tick == QSlider::Both )
    {
	str = "Both";
    }
    
    return str;
}


QSlider::TickSetting GuiTools::QSliderTickSetting( QString	str )
{
    QSlider::TickSetting tick = QSlider::NoMarks;
    
    if( str == "Above" )
    {
	tick = QSlider::Above;
    }
    else if( str == "Left" )
    {
	tick = QSlider::Left;
    }
    else if( str == "Below" )
    {
	tick = QSlider::Below;
    }
    else if( str == "Right" )
    {
	tick = QSlider::Right;
    }
    else if( str == "Both" )
    {
	tick = QSlider::Both;
    }
    
    return tick;
}


const QStrList& GuiTools::GetQDialogSlots( void )
{
    if( itsQDialogSlots.isEmpty() )
    {
	itsQDialogSlots.append( "done(int)" );
	itsQDialogSlots.append( "accept()" );
	itsQDialogSlots.append( "reject()" );
    }
    
    return itsQDialogSlots;
}


QString GuiTools::QTabBarShapeStr(
    QTabBar::Shape	theShape
    )
{
    QString str;
    
    switch( theShape )
    {
	case QTabBar::RoundedAbove:
	{
	    str = "RoundedAbove";
	    break;
	}
	case QTabBar::RoundedBelow:
	{
	    str = "RoundedBelow";
	    break;
	}
	case QTabBar::TriangularAbove:
	{
	    str = "TriangularAbove";
	    break;
	}
	case QTabBar::TriangularBelow:
	{
	    str = "TriangularBelow";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return str;
}


QTabBar::Shape GuiTools::QTabBarShape(
    const QString&	theShapeStr
    )
{
    QTabBar::Shape shape = QTabBar::RoundedAbove;
    
    if( theShapeStr == "RoundedAbove" )
    {
	shape = QTabBar::RoundedAbove;
    }
    else if( theShapeStr == "RoundedBelow" )
    {
	shape = QTabBar::RoundedBelow;
    }
    else if( theShapeStr == "TriangularAbove" )
    {
	shape = QTabBar::TriangularAbove;
    }
    else if( theShapeStr == "TriangularBelow" )
    {
	shape = QTabBar::TriangularBelow;
    }
    
    return shape;
}


QString GuiTools::QScrollViewResizePolicyStr(
    QScrollView::ResizePolicy	thePolicy
    )
{
    QString policyStr;
    
    switch( thePolicy )
    {
	case QScrollView::Default:
	{
	    policyStr = "Default";
	    break;
	}
	case QScrollView::Manual:
	{
	    policyStr = "Manual";
	    break;
	}
	case QScrollView::AutoOne:
	{
	    policyStr = "AutoOne";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return policyStr;
}


QScrollView::ResizePolicy GuiTools::QScrollViewResizePolicy(
    const QString&	thePolicyStr
    )
{
    QScrollView::ResizePolicy policy = QScrollView::Default;
    
    if( thePolicyStr == "Default" )
    {
	policy = QScrollView::Default;
    }
    else if( thePolicyStr == "Manual" )
    {
	policy = QScrollView::Manual;
    }
    else if( thePolicyStr == "AutoOne" )
    {
	policy = QScrollView::AutoOne;
    }
    
    return policy;
}


QString GuiTools::QScrollViewScrollBarModeStr(
    QScrollView::ScrollBarMode	theMode
    )
{
    QString modeStr;
    
    switch( theMode )
    {
	case QScrollView::Auto:
	{
	    modeStr = "Auto";
	    break;
	}
	case QScrollView::AlwaysOff:
	{
	    modeStr = "AlwaysOff";
	    break;
	}
	case QScrollView::AlwaysOn:
	{
	    modeStr = "AlwaysOn";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return modeStr;
}


QScrollView::ScrollBarMode GuiTools::QScrollViewScrollBarMode(
    const QString&	theModeStr
    )
{
    QScrollView::ScrollBarMode mode = QScrollView::Auto;
    
    if( theModeStr == "Auto" )
    {
	mode = QScrollView::Auto;
    }
    else if( theModeStr == "AlwaysOff" )
    {
	mode = QScrollView::AlwaysOff;
    }
    else if( theModeStr == "AlwaysOn" )
    {
	mode = QScrollView::AlwaysOn;
    }
    
    return mode;
}


QString GuiTools::QListViewWidthModeStr(
    QListView::WidthMode	theMode
    )
{
    QString str;
    
    switch( theMode )
    {
	case QListView::Manual:
	{
	    str = "Manual";
	    break;
	}
	case QListView::Maximum:
	{
	    str = "Maximum";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return str;
}


QListView::WidthMode GuiTools::QListViewWidthMode(
    const QString&	theWidthMode
    )
{
    QListView::WidthMode mode = QListView::Maximum;
    
    if( theWidthMode == "Manual" )
    {
	mode = QListView::Manual;
    }
    else if( theWidthMode == "Maximum" )
    {
	mode = QListView::Maximum;
    }
    
    return mode;
}


int GuiTools::numSiblings(
    const QListViewItem*	theItem
    )
{
    // Need to count the item itself
    int numSiblings = 0;

    for( const QListViewItem* sibling = theItem; 
	 sibling;
	 sibling = sibling->nextSibling(), ++numSiblings );
    
    return numSiblings;
}


QString GuiTools::QGridDirectionStr(
    QGrid::Direction    theDirection
    )
{
    QString str;
    
    switch( theDirection )
    {
        case QGrid::Horizontal:
        {
            str = "Horizontal";
            break;
        }
        case QGrid::Vertical:
        {
            str = "Vertical";
            break;
        }
        default:
        {
            break;
        }
    }
    
    return str;
}


QGrid::Direction GuiTools::QGridDirection(
    const QString&      theDirection
    )
{
    QGrid::Direction dir = QGrid::Horizontal;
    
    if( theDirection == "Horizontal" )
    {
        dir = QGrid::Horizontal;
    }
    else if( theDirection == "Vertical" )
    {
        dir = QGrid::Vertical;
    }
    
    return dir;
}


QString GuiTools::ArrowTypeStr(
    Qt::ArrowType	theArrowType
    )
{
    QString str;
    
    switch( theArrowType )
    {
	case Qt::UpArrow:
	{
	    str = "UpArrow";
	    break;
	}
	case Qt::DownArrow:
	{
	    str = "DownArrow";
	    break;
	}
	case Qt::LeftArrow:
	{
	    str = "LeftArrow";
	    break;
	}
	case Qt::RightArrow:
	{
	    str = "RightArrow";
	    break;
	}
    }
    
    return str;
}


Qt::ArrowType GuiTools::ArrowTypeVal(
    const QString&	theArrowTypeStr
    )
{
    Qt::ArrowType type = Qt::UpArrow;
    
    if( theArrowTypeStr == "UpArrow" )
    {
	type = Qt::UpArrow;
    }
    else if( theArrowTypeStr == "DownArrow" )
    {
	type = Qt::DownArrow;
    }
    else if( theArrowTypeStr == "LeftArrow" )
    {
	type = Qt::LeftArrow;
    }
    else if( theArrowTypeStr == "RightArrow" )
    {
	type = Qt::RightArrow;
    }
    
    return type;
}
