/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GuiTools.h,v 2.6 1999/10/16 01:43:46 jeff Exp $

*******************************************************************************/


#ifndef GuiTools_included
#define GuiTools_included

#include <qcombo.h>
#include <qdrawutil.h>
#include <qfont.h>
#include <qgrid.h>
#include <qlcdnum.h>
#include <qlined.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qscrollview.h>
#include <qslider.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qtabbar.h>
#include <qwidget.h>

class GuiTools
{
public:

    GuiTools();
    ~GuiTools();
    // the ctor and dtor are declared but never implemented


    //
    // QApplication
    //

    static const QString platinumStyle;
    static const QString cdeStyle;
    static const QString motifStyle;
    static const QString windowsStyle;

    static void SetStyle( const QString& style );
    // Set the application's style to the one specified.

    //
    // QWidget
    //

    static void SetVisible( QWidget*	widget,
		       bool	visible );
    // show or hide the widget

    static QRect GetGlobalGeometry
    (
	const QWidget*	widget
    );
    // get the geometry rect of the widget in world coordinates

    static void ChangeColor( 
        QWidget*        widget,
        const QColor&   color
        );
    // Change the color of the given widget to the specified color

    static QString QWidgetFocusPolicyStr(
	QWidget::FocusPolicy	theFocusPolicy
	);
    // Return a string value for the focus policy

    static QWidget::FocusPolicy QWidgetFocusPolicy(
	const QString&	theFocusPolicyStr
	);
    // Return a focus policy enum for the string, defaulting to NoFocus

    static QString QWidgetBackgroundModeStr(
	QWidget::BackgroundMode	theBackgroundMode
	);
    // Return a string value for the background mode

    static QWidget::BackgroundMode QWidgetBackgroundMode(
	const QString&	theBackgroundModeStr
	);
    // Return a background mode enum for the string, defaulting to
    // palette background.

    static QString QWidgetPropagationModeStr(
	QWidget::PropagationMode	thePropagationMode
	);
    // Return a string value for the propagation mode.

    static QWidget::PropagationMode QWidgetPropagationMode(
	const QString&	thePropagationModeStr
	);
    // Return a propagation mode enum for the string, defaulting to no
    // children

    static QString AlignmentStr(
	int	theAlignment
	);
    // Return a string value for the given alignment

    static int Alignment(
	const QString&	theAlignmentStr
	);
    // Return an alignment value for the given alignment string

    //
    // QListBox
    //

    static void TransferListBox( const QListBox*	source,
				 QListBox*		dest );
    // transfer list from the source list box to the destination


    //
    // QComboBox
    //

    static void TransferListFromCombo( const QComboBox*	source,
				       QListBox*	dest );
    // transfer list from the source combo box to the list box

    static void TransferListToCombo( QComboBox*		dest,
				     const QListBox*	source );
    // transfer list from the source list box to the combo box


    //
    // QLCDNumber
    //

    static QString QLCDNumberSegStyleStr( QLCDNumber::SegmentStyle style );
    // return a string value for the style enumeration

    static QLCDNumber::SegmentStyle QLCDNumberSegStyle( QString	str );
    // return a segment style for the string, default to Outline


    //
    // QLineEdit
    //

    static QString QLineEditEchoModeStr( QLineEdit::EchoMode	mode );
    // return a string value for the echo mode enumeration

    static QLineEdit::EchoMode QLineEditEchoMode( QString	str );
    // return a echo mode for the string, default to Normal


    //
    // QSlider
    //

    static QString QSliderTickSettingStr( QSlider::Orientation	orient,
					  QSlider::TickSetting	tick );
    // return a string value for the tick setting

    static QSlider::TickSetting QSliderTickSetting( QString	str );
    // return a tick setting for the string, default to NoMarks


    //
    // QDialog
    //
    
    static const QStrList& GetQDialogSlots( void );
    // Return a list of slots that are defined by a QDialog


    //
    // QFont
    //

    static QString QFontStyleHintStr( QFont::StyleHint	hint );
    // return a string value for the enumeration

    static QFont::StyleHint QFontStyleHint( QString	str );
    // return the style hint for the string, default to Helvetica

    static QString QFontWeightStr( int	weight );
    // return a string value for the wieght

    static QString QFontCharSetStr( QFont::CharSet	set );
    // return a string value for the set

    static QFont::CharSet QFontCharSet( QString		set );
    // return the char set for the string, default to Latin1


    //
    // QStrList
    //

    static void QStrListAppendAll( QStrList&		dest,
				   const QStrList&	source );
    // append all the items in the source to the destination

    
    //
    // QTabBar
    //

    static QString QTabBarShapeStr( 
	QTabBar::Shape	theShape
	);
    // Return a string for the shape enum

    static QTabBar::Shape QTabBarShape(
	const QString&	theShapeStr
	);
    // Return an enum for the shape, defaulting to RoundedAbove

    //
    // QScrollView
    //

    static QString QScrollViewResizePolicyStr(
	QScrollView::ResizePolicy	thePolicy
	);
    // Return a string for the policy enum

    static QScrollView::ResizePolicy QScrollViewResizePolicy(
	const QString&	thePolicyStr
	);
    // Return an enum for the policy, defaulting to Default

    static QString QScrollViewScrollBarModeStr(
	QScrollView::ScrollBarMode	theMode
	);
    // Return a string for the scroll bar mode

    static QScrollView::ScrollBarMode QScrollViewScrollBarMode(
	const QString&	theModeStr
	);
    // Return an enum for the scroll bar mode

    //
    // QListView
    //

    static QString QListViewWidthModeStr(
	QListView::WidthMode	theMode
	);
    // Return a string for the width mode enum

    static QListView::WidthMode QListViewWidthMode(
	const QString&	theWidthMode
	);
    // Return an enum for the policy, defaulting to Maximum

    static int numSiblings(
	const QListViewItem*	theItem
	);
    // Count how many siblings this item has.  This item should be the 
    // first sibling.


    //
    // QGrid
    //

    static QString QGridDirectionStr( 
        QGrid::Direction        theDirection
        );
    // Return a string for the direction enum

    static QGrid::Direction QGridDirection(
        const QString&  theDirection
        );
    // Return an enum for the direction, defaulting to Horizontal


    //
    // Misc
    //

    static QString ArrowTypeStr(
	Qt::ArrowType	theArrowType
	);
    // Return a string for the arrow type

    static Qt::ArrowType ArrowTypeVal(
	const QString&	theArrowTypeStr
	);
    // Return an ArrowType or the arrow type

private:

    GuiTools( const GuiTools& );
    const GuiTools& operator = ( const GuiTools& );
    
    // no one should ever declare a GuiTools, just use static members

    static QStrList	itsQDialogSlots;
    // slots defined by a QDialog
};


#endif // GuiTools_included
