/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridSpacer.cpp,v 2.2 1999/10/16 01:45:08 jeff Exp $

*******************************************************************************/


#include "layout/GridSpacer.h"
#include "layout/GridSpacerPropPage.h"

#define Inherited LayoutItem

GridSpacer::GridSpacer(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridSpacer, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridSpacer_bmp ) )
{
}


GridSpacer::GridSpacer(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridSpacer, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridSpacer_bmp ) )
{
}


GridSpacer::~GridSpacer()
{
}


LayoutItemPropPage* GridSpacer::GetPropertyPage( 
    QWidget*		parent,
    bool		/*topLevel*/,
    const WidgetLayout&	/*theWidgetLayout*/
    )
{
    return new GridSpacerPropPage( this,                                         parent );
}


void GridSpacer::AddToLayout( QGridLayout*	/*theGridLayout*/,
			      int		/*theRowNum*/,
			      int		/*theColNum*/ )
{
    // Do nothing, we're just blank space
}


void GridSpacer::SaveContents( QTextStream&	theStream,
			       const QString&	theId ) const
{
    OutputItemIdentifier( theStream, theId );

    // No properties

    theStream << "}\n";
}


bool GridSpacer::RestoreKeyValue( const QString&	/*theKey*/,
				  const QString&	/*theValue*/,
				  const DlgWidget*	/*theDlgWidget*/ )
{
    // No properties
    
    return FALSE;
}


void GridSpacer::GenerateSourceInGrid( QTextStream&	/*theStream*/,
				       const QString&	/*theLayoutVarName*/,
				       const QString&	/*theLayoutId*/,
				       unsigned int	/*theRowNum*/,
				       unsigned int	/*theColNum*/,
				       const QString&	/*theLayoutParentWidget*/ ) const
{
    // No source to generate
}


QString GridSpacer::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Grid Spacer" );
    return treeLabel;
}
