/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: main.cpp,v 2.4 1999/10/16 01:44:33 jeff Exp $

*******************************************************************************/


#include <qapp.h>
#include <qfont.h>
#include <qregexp.h>
#include <qstring.h>
#include <qtranslator.h>

#include <stdlib.h>
#include <stdio.h>

#include "GuiTools.h"
#include "WidgetModuleMgr.h"
#include "ProjectWnd.h"


int main( int argc, char** argv )
{
    // Load any widget modules
    WidgetModuleMgr::loadModules();

    QApplication* a = WidgetModuleMgr::getQApplication( argc, argv );
    if( !a )
    {
	a = new QApplication( argc, argv );
    }

    // Load the translation file that was configured.
    QString trfile = ProjectWnd::GetAppConfig().readEntry( "TRFile", "" );
    QTranslator translator( 0 );

    if( !trfile.isEmpty() )
    {
        if( translator.load( trfile ) )
        {
            a->installTranslator( &translator );
        }
        else
        {
            cerr << "Failed to load tr file: " << trfile << endl;
        }
    }
    
    QFont font( "helvetica", 12 );
    font.setStyleHint( QFont::Helvetica );
    a->setFont( font, TRUE );

    ProjectWnd* wnd = new ProjectWnd;

    a->setMainWidget( wnd );

    GuiTools::SetStyle( 
        ProjectWnd::GetAppConfig().readEntry( "WidgetStyle",
                                              GuiTools::platinumStyle ) );

    // Remove TMPDIR from environment
    QString envTmpDir = getenv( "TMPDIR" );
    putenv( "TMPDIR" );

    QString tempDir = getenv( "DLGEDIT" );

    if( tempDir.isEmpty() )
	tempDir = "/tmp";

    char *tmpName = tempnam( tempDir, "dlg" );
    wnd->SetTempFile( tmpName ); 

    // Reinsert TMPDIR
    if( ! envTmpDir.isEmpty() )
    {
	envTmpDir = "TMPDIR=" + envTmpDir;
	putenv( (char*)envTmpDir.data() );
    }

    bool runExec = TRUE;
    
    if( a->argc() > 1 )
    {
	if( QString(a->argv()[1]) == "-g" )
	{
	    wnd->AutoGenerate();
	    
	    // don't go into main event loop
	    runExec = FALSE;	    
	}
	else // load dialogs on command line
	{
	    wnd->show();

	    for( int i = 1; i < a->argc(); i++ )
		wnd->ObjectLoad( a->argv()[i] );
	}
    }
    else
    {
	wnd->show();
    }

    int retVal = runExec ? a->exec() : 0;

    // Clean up
    remove( tmpName );

    free( tmpName );

    delete a;

    return retVal;
}


