/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ListBoxPage.cpp,v 2.3 1999/10/16 01:49:02 jeff Exp $

*******************************************************************************/


#include <qlistbox.h>

#include "ListBoxPage.h"

#include "widget/DlgWidget.h"

#include "GuiTools.h"

#define Inherited ListBoxPageData

ListBoxPage::ListBoxPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    const QListBox* list = (QListBox*)widget->GetWidget();
    
    connect( itsAddButton, SIGNAL(clicked()), SLOT(AddClicked()) );
    connect( itsChangeButton, SIGNAL(clicked()), SLOT(ChangeClicked()) );
    connect( itsDeleteButton, SIGNAL(clicked()), SLOT(DeleteClicked()) );
    connect( itsList, SIGNAL(highlighted(int)), 
	     SLOT(ListSelChanged(int)) );
    connect( itsNewItem, SIGNAL(textChanged(const QString&)),
	     SLOT(NewItemChanged(const QString&)) );

    GuiTools::TransferListBox( list, itsList );

    isDragSelect->setChecked( list->dragSelect() );
    isAutoScroll->setChecked( list->autoScroll() );
    isScrollbar->setChecked( list->scrollBar() );
    isAutoScrollbar->setChecked( list->autoScrollBar() );
    isBottomScrollbar->setChecked( list->bottomScrollBar() );
    isAutoBottomScrollbar->setChecked( list->autoBottomScrollBar() );
    isSmoothScrolling->setChecked( list->smoothScrolling() );
    isMultiSelection->setChecked( list->isMultiSelection() );
    isAutoUpdate->setChecked( list->autoUpdate() );
    
    Update();
}


ListBoxPage::~ListBoxPage()
{
}


void ListBoxPage::Apply( DlgWidget*	widget )
{
    QListBox* list = (QListBox*)widget->GetWidget();
    
    GuiTools::TransferListBox( itsList, list );

    list->setDragSelect( isDragSelect->isChecked() );
    list->setAutoScroll( isAutoScroll->isChecked() );
    list->setScrollBar( isScrollbar->isChecked() );
    list->setAutoScrollBar( isAutoScrollbar->isChecked() );
    list->setBottomScrollBar( isBottomScrollbar->isChecked() );
    list->setAutoBottomScrollBar( isAutoBottomScrollbar->isChecked() );
    list->setSmoothScrolling( isSmoothScrolling->isChecked() );
    list->setMultiSelection( isMultiSelection->isChecked() );
    list->setAutoUpdate( isAutoUpdate->isChecked() );
}


void ListBoxPage::AddClicked()
{
    const char* text = itsNewItem->text();
    if( text && text[0] )
    {
	itsList->insertItem( text );
    }
}


void ListBoxPage::ChangeClicked()
{
    const char* text = itsNewItem->text();
    if( text && text[0] )
    {
	itsList->changeItem( text, itsList->currentItem() );
    }
}


void ListBoxPage::DeleteClicked()
{
    int item = itsList->currentItem();
    if( item >= 0 )
    {
	itsList->removeItem( item );
	Update();
    }
}


void ListBoxPage::ListSelChanged( int item )
{
    itsNewItem->setText( itsList->text( item ) );
    Update();
}


void ListBoxPage::NewItemChanged( const QString& )
{
    Update();
}


void ListBoxPage::Update()
{
    const char* text = itsNewItem->text();
    bool newText = ( text && text[0] );
    bool itemSelected = (itsList->currentItem() != -1);

    itsAddButton->setEnabled( newText );
    itsChangeButton->setEnabled( newText & itemSelected );
    itsDeleteButton->setEnabled( itemSelected );
}


