/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: TextBrowserPage.cpp,v 2.2 1999/10/16 01:49:49 jeff Exp $

*******************************************************************************/


#include <qtextbrowser.h>
#include <qstring.h>

#include "TextBrowserPage.h"

#include "widget/DlgWidget.h"
#include "widget/DlgTextBrowser.h"

#define Inherited TextBrowserPageData

TextBrowserPage::TextBrowserPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
  
    QTextBrowser* tBrowser = (QTextBrowser*)widget->GetWidget();
    
    DlgTextBrowser *dlgTBrowser = dynamic_cast<DlgTextBrowser*>( widget );
    if( dlgTBrowser )
    {
      bool useSrc = dlgTBrowser->SourceUsed();
      if( useSrc )
      {
        itsText->setEnabled( false );
        itsSource->setText( dlgTBrowser->GetSource() );
      } else
      {
        itsSource->setEnabled( false );
        itsText->setText( tBrowser->text() );
      }

      useSource->setChecked( useSrc );
      useText->setChecked( ! useSrc );
    }
}

TextBrowserPage::~TextBrowserPage()
{
}

void TextBrowserPage::UseTextOrSource( bool uText )
{
  itsText->setEnabled( uText );
  itsSource->setEnabled( ! uText );
}

void TextBrowserPage::Apply( DlgWidget*	widget )
{
  QTextBrowser* tBrowser = (QTextBrowser*)widget->GetWidget();
 
  DlgTextBrowser *dlgTBrowser = dynamic_cast<DlgTextBrowser*>( widget );
  
  if( dlgTBrowser )
  {

    if( useText->isChecked() )
    {
      tBrowser->setText( itsText->text() );
    } else
    {
      tBrowser->setText( "" );
      dlgTBrowser->SetSource( itsSource->text() );
    }

    dlgTBrowser->UseSource( ! (useText->isChecked()) );

  }      
}

