/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetCommonPage.cpp,v 2.2 1999/10/16 01:50:06 jeff Exp $

*******************************************************************************/


#include "WidgetCommonPage.h"

#include "widget/DlgWidget.h"

#define Inherited WidgetCommonPageData

WidgetCommonPage::WidgetCommonPage
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name )
{
    QWidget* wid = widget->GetWidget();
    
    itsName->setText( wid->name() );
    itsCaption->setText( wid->caption() );

    unsigned int policyIndex = 0;
    switch( wid->focusPolicy() )
    {
	case QWidget::TabFocus:
	{
	    policyIndex = Policy_Tab;
	    break;
	}
	case QWidget::ClickFocus:
	{
	    policyIndex = Policy_Click;
	    break;
	}
	case QWidget::StrongFocus:
	{
	    policyIndex = Policy_Strong;
	    break;
	}
	case QWidget::NoFocus:
	{
	    policyIndex = Policy_No;
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    itsFocusPolicy->setCurrentItem( policyIndex );

#define CASE_BACKGROUND_MODE( mode ) \
    case QWidget::mode: backgroundIndex = Background_##mode; break;

    unsigned int backgroundIndex = Background_PaletteBackground;
    switch( wid->backgroundMode() )
    {
	CASE_BACKGROUND_MODE( FixedColor )
	CASE_BACKGROUND_MODE( FixedPixmap )
	CASE_BACKGROUND_MODE( NoBackground )
	CASE_BACKGROUND_MODE( PaletteForeground )
	CASE_BACKGROUND_MODE( PaletteButton )
	CASE_BACKGROUND_MODE( PaletteLight )
	CASE_BACKGROUND_MODE( PaletteMidlight )
	CASE_BACKGROUND_MODE( PaletteDark )
	CASE_BACKGROUND_MODE( PaletteMid )
	CASE_BACKGROUND_MODE( PaletteText )
	CASE_BACKGROUND_MODE( PaletteBrightText )
	CASE_BACKGROUND_MODE( PaletteBase )
	CASE_BACKGROUND_MODE( PaletteBackground )
	CASE_BACKGROUND_MODE( PaletteShadow )
	CASE_BACKGROUND_MODE( PaletteHighlight )
	CASE_BACKGROUND_MODE( PaletteHighlightedText )
    }
    
    itsBackgroundMode->setCurrentItem( backgroundIndex );

    unsigned int fontIndex = 0;
    switch( wid->fontPropagation() )
    {
	case QWidget::NoChildren:
	{
	    fontIndex = Font_NoChildren;
	    break;
	}
	case QWidget::SameFont:
	{
	    fontIndex = Font_SameFont;
	    break;
	}
	case QWidget::AllChildren:
	{
	    fontIndex = Font_AllChildren;
	    break;
	}
    }
    
    itsFontPropagation->setCurrentItem( fontIndex );
    
    unsigned int paletteIndex = 0;
    switch( wid->palettePropagation() )
    {
	case QWidget::NoChildren:
	{
	    paletteIndex = Palette_NoChildren;
	    break;
	}
	case QWidget::SamePalette:
	{
	    paletteIndex = Palette_SamePalette;
	    break;
	}
	case QWidget::AllChildren:
	{
	    paletteIndex = Palette_AllChildren;
	    break;
	}
    }
    
    itsPalettePropagation->setCurrentItem( paletteIndex );

    isEnabled->setChecked( widget->IsEnabled() );
}


WidgetCommonPage::~WidgetCommonPage()
{
}


void WidgetCommonPage::Apply( DlgWidget*	widget )
{
    QWidget* wid = widget->GetWidget();
    
    wid->setName( itsName->text() );
    wid->setCaption( itsCaption->text() );

    QWidget::FocusPolicy focusPolicy;
    switch( itsFocusPolicy->currentItem() )
    {
	case Policy_Tab:
	{
	    focusPolicy = QWidget::TabFocus;
	    break;
	}
	case Policy_Click:
	{
	    focusPolicy = QWidget::ClickFocus;
	    break;
	}
	case Policy_Strong:
	{
	    focusPolicy = QWidget::StrongFocus;
	    break;
	}
	case Policy_No:
	{
	    focusPolicy = QWidget::NoFocus;
	    break;
	}
    }
    
    wid->setFocusPolicy( focusPolicy );

#define CASE_BACKGROUND( mode ) \
    case Background_##mode: backgroundMode = QWidget::mode; break;

    QWidget::BackgroundMode backgroundMode = QWidget::PaletteBackground;
    bool setBackground = TRUE;
    switch( itsBackgroundMode->currentItem() )
    {
	case Background_FixedColor:
	case Background_FixedPixmap:
	{
	    // These items can not be passed to setBackgroundMode
	    setBackground = FALSE;

	    break;
	}
	CASE_BACKGROUND( NoBackground )
	CASE_BACKGROUND( PaletteForeground )
	CASE_BACKGROUND( PaletteButton )
	CASE_BACKGROUND( PaletteLight )
	CASE_BACKGROUND( PaletteMidlight )
	CASE_BACKGROUND( PaletteDark )
	CASE_BACKGROUND( PaletteMid )
	CASE_BACKGROUND( PaletteText )
	CASE_BACKGROUND( PaletteBrightText )
	CASE_BACKGROUND( PaletteBase )
	CASE_BACKGROUND( PaletteBackground )
	CASE_BACKGROUND( PaletteShadow )
	CASE_BACKGROUND( PaletteHighlight )
	CASE_BACKGROUND( PaletteHighlightedText )
    }
    
    if( setBackground )
    {
	wid->setBackgroundMode( backgroundMode );
    }

    QWidget::PropagationMode propMode = QWidget::NoChildren;
    switch( itsFontPropagation->currentItem() )
    {
	case Font_NoChildren:
	{
	    propMode = QWidget::NoChildren;
	    break;
	}
	case Font_SameFont:
	{
	    propMode = QWidget::SameFont;
	    break;
	}
	case Font_AllChildren:
	{
	    propMode = QWidget::AllChildren;
	    break;
	}
    }
    
    wid->setFontPropagation( propMode );
    
    switch( itsPalettePropagation->currentItem() )
    {
	case Palette_NoChildren:
	{
	    propMode = QWidget::NoChildren;
	    break;
	}
	case Palette_SamePalette:
	{
	    propMode = QWidget::SamePalette;
	    break;
	}
	case Palette_AllChildren:
	{
	    propMode = QWidget::AllChildren;
	    break;
	}
    }
    
    wid->setPalettePropagation( propMode );

    widget->SetEnabled( isEnabled->isChecked() );
}
