/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgButton.cpp,v 2.3 1999/10/16 01:50:25 jeff Exp $

*******************************************************************************/


#include "widget/DlgButton.h"

#include "prop/ButtonPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgButton::itsSignals;
int DlgButton::itsNextWidgetNum = 1;

DlgButton::DlgButton() :
    Inherited( DlgWidget::DW_Button )
{
}


DlgButton::DlgButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgButton::~DlgButton()
{
}


void DlgButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ButtonPage( this, parent, "Button" ) );

    labels.append( tr( "Button" ) );
}


void DlgButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Button {" << endl;
    }
  
    QButton* widget = GetButton();

    stream << saveIndent <<
	"Text {" << widget->text() << "}" << endl;

    stream << saveIndent <<
	"AutoRepeat {";
    SaveBool( stream, widget->autoRepeat() );
    stream << "}" << endl;
    
    stream << saveIndent <<
	"AutoResize {";
    SaveBool( stream, widget->autoResize() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QButton* widget = GetButton();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "AutoRepeat" )
    {
	widget->setAutoRepeat( (value == "TRUE") );
    }
    else if( key == "AutoResize" )
    {
	widget->setAutoResize( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QButton* widget = GetButton();

    // Don't call setText when the user wants a background pixmap.
    if( !UseBackgroundPixmap() )
    {
	stream << varName << "setText( ";
        GenerateText( widget->text(), widgetLayout, stream );
        stream << " );\n";
    }

    stream << varName << "setAutoRepeat( " << 
	(widget->autoRepeat() ? "TRUE" : "FALSE") << " );\n";
    stream << varName << "setAutoResize( " <<
	(widget->autoResize() ? "TRUE" : "FALSE") << " );\n";
}


void DlgButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "pressed()" );
	itsSignals.append( "released()" );
	itsSignals.append( "clicked()" );
	itsSignals.append( "toggled(bool)" );
    }

    return itsSignals;
}


void DlgButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QButton* widget = 
	new QButton( theParent,
		     CreateName( "Button", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
