/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgCheckBox.cpp,v 2.3 1999/10/16 01:50:30 jeff Exp $

*******************************************************************************/


#include "widget/DlgCheckBox.h"

#include "prop/CheckBoxPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgButton

QStrList DlgCheckBox::itsSignals;
int DlgCheckBox::itsNextWidgetNum = 1;

DlgCheckBox::DlgCheckBox() :
    Inherited( DlgWidget::DW_CheckBox )
{
}


DlgCheckBox::DlgCheckBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgCheckBox::~DlgCheckBox()
{
}


void DlgCheckBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new CheckBoxPage( this, parent, "CheckBox" ) );

    labels.append( tr( "Check Box" ) );
}


void DlgCheckBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "CheckBox {" << endl;
    }
  
    QCheckBox* widget = GetCheckBox();

    stream << saveIndent << "Checked {";
    SaveBool( stream, widget->isChecked() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgCheckBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QCheckBox* widget = GetCheckBox();

    // Compare to known keys
    if( key == "Checked" )
    {
	widget->setChecked( (value == "TRUE") );
    } 
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgCheckBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QCheckBox* widget = GetCheckBox();

    stream << varName << "setChecked( " <<
	(widget->isChecked() ? "TRUE" : "FALSE") << " );\n";
}


void DlgCheckBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgCheckBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgCheckBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QCheckBox* widget = 
	new QCheckBox( theParent,
		       CreateName( "CheckBox", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
