/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgFrame.cpp,v 2.3 1999/10/16 01:50:34 jeff Exp $

*******************************************************************************/


#include "widget/DlgFrame.h"

#include "prop/FramePage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgFrame::itsSignals;
int DlgFrame::itsNextWidgetNum = 1;

DlgFrame::DlgFrame() :
    Inherited( DlgWidget::DW_Frame )
{
}


DlgFrame::DlgFrame(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgFrame::~DlgFrame()
{
}


void DlgFrame::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new FramePage( this, parent, "Frame" ) );

    labels.append( tr( "Frame" ) );
}


void DlgFrame::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Frame {" << endl;
    }
  
    QFrame* widget = GetFrame();

    stream << saveIndent <<
	"Style {" << widget->frameStyle() << "}" << endl;
    
    stream << saveIndent <<
	"LineWidth {" << widget->lineWidth() << "}" << endl;
    
    stream << saveIndent << 
	"MidLineWidth {" << widget->midLineWidth() << "}" << endl;

    stream << saveIndent <<
	"FrameMargin {" << widget->margin() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgFrame::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QFrame* widget = GetFrame();

    // Compare to known keys
    if( key == "Style" )
    {
	widget->setFrameStyle( value.toInt() );
    } 
    else if( key == "LineWidth" )
    {
	widget->setLineWidth( value.toInt() );
    }
    else if( key == "MidLineWidth" )
    {
	widget->setMidLineWidth( value.toInt() );
    }
    else if( key == "FrameMargin" )
    {
	widget->setMargin( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgFrame::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QFrame* widget = GetFrame();

    stream << varName <<
	"setFrameStyle( " << widget->frameStyle() << " );" << endl;
    
    stream << varName <<
	"setLineWidth( " << widget->lineWidth() << " );" << endl;
    
    stream << varName <<
	"setMidLineWidth( " << widget->midLineWidth() << " );" << endl;

    stream << varName <<
	"QFrame::setMargin( " << widget->margin() << " );" << endl;
}


void DlgFrame::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgFrame::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgFrame::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QFrame* widget = 
	new QFrame( theParent,
		    CreateName( "Frame", 
				theName, 
				itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
