/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgGrid.cpp,v 2.4 1999/10/16 01:50:36 jeff Exp $

*******************************************************************************/


#include "widget/DlgGrid.h"

#include "prop/GridPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgGrid::itsSignals;
int DlgGrid::itsNextWidgetNum = 1;

DlgGrid::DlgGrid() :
    Inherited( DlgWidget::DW_Grid ),
    myDirection( QGrid::Horizontal ),
    myNumRowsCols( 5 ),
    mySpacing( 5 )
{
}


DlgGrid::DlgGrid(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myDirection( QGrid::Horizontal ),
    myNumRowsCols( 5 ),
    mySpacing( 5 )
{
}


DlgGrid::~DlgGrid()
{
}


void DlgGrid::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new GridPage( this, parent, "Grid" ) );

    labels.append( tr( "Grid" ) );
}


void DlgGrid::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Grid {" << endl;
    }
  
    stream << saveIndent << "Direction {" 
           << GuiTools::QGridDirectionStr( myDirection ) << "}" << endl;
    
    stream << saveIndent << "RowsCols {" << myNumRowsCols << "}" << endl;

    stream << saveIndent << "Spacing {" << mySpacing << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgGrid::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Direction" )
    {
        SetDirection( GuiTools::QGridDirection( value ) );
    }
    else if( key == "RowsCols" )
    {
        SetNumRowsCols( value.toInt() );
    }
    else if( key == "Spacing" )
    {
        SetSpacing( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgGrid::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    stream << varName << "setSpacing( " << mySpacing << " );" << endl;
}


void DlgGrid::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << myNumRowsCols << ", " 
           << "QGrid::" << GuiTools::QGridDirectionStr( myDirection ) << ", "
           << parentName << ", " << name;
}


void DlgGrid::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgGrid::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgGrid::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QGrid* widget = 
	new QGrid( myNumRowsCols,
                   myDirection,
                   theParent,
                   CreateName( "Grid", 
                               theName, 
                               itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );

    SetSpacing( mySpacing );
}
