/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLCDNumber.cpp,v 2.3 1999/10/16 01:50:45 jeff Exp $

*******************************************************************************/


#include "widget/DlgLCDNumber.h"

#include "prop/LCDNumberPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgLCDNumber::itsSignals;
int DlgLCDNumber::itsNextWidgetNum = 1;

DlgLCDNumber::DlgLCDNumber() :
    Inherited( DlgWidget::DW_LCDNumber )
{
}


DlgLCDNumber::DlgLCDNumber(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgLCDNumber::~DlgLCDNumber()
{
}


void DlgLCDNumber::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LCDNumberPage( this, parent, "LCDNumber" ) );

    labels.append( tr( "LCD Number" ) );
}


void DlgLCDNumber::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "LCDNumber {" << endl;
    }
  
    QLCDNumber* widget = GetLCDNumber();

    stream << saveIndent <<
	"Digits {" << widget->numDigits() << "}" << endl;
    
    stream << saveIndent << "SmallDecimalPoint {";
    SaveBool( stream, widget->smallDecimalPoint() );
    stream << "}" << endl;
    
    stream << saveIndent << "Mode {";
    switch( widget->mode() )
    {
	case QLCDNumber::HEX:
	{
	    stream << "Hex";
	    break;
	}
	case QLCDNumber::DEC:
	{
	    stream << "Dec";
	    break;
	}
	case QLCDNumber::OCT:
	{
	    stream << "Oct";
	    break;
	}
	case QLCDNumber::BIN:
	{
	    stream << "Bin";
	    break;
	}
    }
    stream << "}" << endl;
    
    stream << saveIndent << "SegmentStyle {" << 
	GuiTools::QLCDNumberSegStyleStr( widget->segmentStyle() ) << 
	"}" << endl;
    

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgLCDNumber::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLCDNumber* widget = GetLCDNumber();

    // Compare to known keys
    if( key == "Digits" )
    {
	widget->setNumDigits( value.toInt() );
    } 
    else if( key == "SmallDecimalPoint" )
    {
	widget->setSmallDecimalPoint( (value == "TRUE") );
    }
    else if( key == "Mode" )
    {
	if( value == "Hex" )
	{
	    widget->setHexMode();
	}
	else if( value == "Dec" )
	{
	    widget->setDecMode();
	}
	else if( value == "Oct" )
	{
	    widget->setOctMode();
	}
	else if( value == "Bin" )
	{
	    widget->setBinMode();
	}
    }
    else if( key == "SegmentStyle" )
    {
	widget->setSegmentStyle( GuiTools::QLCDNumberSegStyle( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgLCDNumber::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLCDNumber* widget = GetLCDNumber();

    stream << varName << "setSmallDecimalPoint( " <<
	(widget->smallDecimalPoint() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName <<
	"setNumDigits( " << widget->numDigits() << " );" << endl;
    
    stream << varName << "setMode( QLCDNumber::";
    switch( widget->mode() )
    {
	case QLCDNumber::HEX:
	{
	    stream << "HEX";
	    break;
	}
	case QLCDNumber::OCT:
	{
	    stream << "OCT";
	    break;
	}
	case QLCDNumber::BIN:
	{
	    stream << "BIN";
	    break;
	}
	case QLCDNumber::DEC:
	{
	    stream << "DEC";
	    break;
	}
    }
    stream << " );" << endl;
    
    stream << varName << "setSegmentStyle( QLCDNumber::" <<
	GuiTools::QLCDNumberSegStyleStr( widget->segmentStyle() ) << 
	" );" << endl;
}


void DlgLCDNumber::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLCDNumber::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "overflow()" );
    }

    return itsSignals;
}


void DlgLCDNumber::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QLCDNumber* widget = 
	new QLCDNumber( theParent,
			CreateName( "LCDNumber", 
				    theName, 
				    itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
