/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLabel.cpp,v 2.3 1999/10/16 01:50:48 jeff Exp $

*******************************************************************************/


#include "widget/DlgLabel.h"

#include "prop/LabelPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgLabel::itsSignals;
int DlgLabel::itsNextWidgetNum = 1;

DlgLabel::DlgLabel() :
    Inherited( DlgWidget::DW_Label )
{
}


DlgLabel::DlgLabel(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgLabel::~DlgLabel()
{
}


void DlgLabel::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LabelPage( this, parent, "Label" ) );

    labels.append( tr( "Label" ) );
}


void DlgLabel::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Label {" << endl;
    }
  
    QLabel* widget = GetLabel();

    stream << saveIndent <<
	"Text {" << widget->text() << "}" << endl;
    
    stream << saveIndent << "Alignment {" 
           << GuiTools::AlignmentStr( widget->alignment() ) << "}" << endl;
    
    stream << saveIndent << "AutoResize {";
    SaveBool( stream, widget->autoResize() );
    stream << "}" << endl;
    
    stream << saveIndent <<
	"Margin {" << widget->margin() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgLabel::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLabel* widget = GetLabel();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "Alignment" )
    {
        bool ok = FALSE;
        int alignment = value.toInt( &ok );
        if( !ok )
        {
            alignment = GuiTools::Alignment( value );
        }
        
	widget->setAlignment( alignment );
    }
    else if( key == "AutoResize" )
    {
	widget->setAutoResize( (value == "TRUE") );
    }
    else if( key == "Margin" )
    {
	widget->setMargin( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgLabel::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLabel* widget = GetLabel();

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setAlignment( " 
           << GuiTools::AlignmentStr( widget->alignment() ) << " );" << endl;
    
    stream << varName << "setMargin( " << widget->margin() << " );" << endl;
}


void DlgLabel::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLabel::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgLabel::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QLabel* widget = 
	new QLabel( theParent,
		    CreateName( "Label", 
				theName, 
				itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
