/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLineEdit.cpp,v 2.5 1999/10/16 01:50:50 jeff Exp $

*******************************************************************************/


#include "widget/DlgLineEdit.h"

#include "prop/LineEditPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgLineEdit::itsSignals;
int DlgLineEdit::itsNextWidgetNum = 1;

DlgLineEdit::DlgLineEdit() :
    Inherited( DlgWidget::DW_LineEdit )
{
}


DlgLineEdit::DlgLineEdit(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgLineEdit::~DlgLineEdit()
{
}


void DlgLineEdit::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LineEditPage( this, parent, "LineEdit" ) );

    labels.append( tr( "Line Edit" ) );
}


void DlgLineEdit::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "LineEdit {" << endl;
    }
  
    QLineEdit* widget = GetLineEdit();

    stream << saveIndent <<
	"Text {" << widget->text() << "}" << endl;
    
    stream << saveIndent << "EchoMode {" <<
	GuiTools::QLineEditEchoModeStr( widget->echoMode() ) << "}" << endl;
    
    stream << saveIndent <<
	"MaxLength {" << widget->maxLength() << "}" << endl;
    
    stream << saveIndent << "FrameShown {";
    SaveBool( stream, widget->frame() );
    stream << "}" << endl;

    stream << saveIndent <<
        "Alignment {" <<
        GuiTools::AlignmentStr( widget->alignment() ) <<
        "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgLineEdit::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLineEdit* widget = GetLineEdit();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "MaxLength" )
    {
	widget->setMaxLength( value.toInt() );
    }
    else if( key == "EchoMode" )
    {
	widget->setEchoMode( GuiTools::QLineEditEchoMode( value ) );
    }
    else if( key == "FrameShown" )
    {
	widget->setFrame( (value == "TRUE") );
    }
    else if( key == "Alignment" )
    {
        widget->setAlignment( GuiTools::Alignment( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgLineEdit::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLineEdit* widget = GetLineEdit();

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << 
	"setMaxLength( " << widget->maxLength() << " );" << endl;
    
    stream << varName << "setFrame( QLineEdit::" <<
	GuiTools::QLineEditEchoModeStr( widget->echoMode() ) << " );" << endl;
    
    stream << varName << "setFrame( " <<
	(widget->frame() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAlignment( " <<
        GuiTools::AlignmentStr( widget->alignment() ) << " );" << endl;
}


void DlgLineEdit::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLineEdit::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "textChanged(const QString&)" );
	itsSignals.append( "returnPressed()" );
    }

    return itsSignals;
}


void DlgLineEdit::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QLineEdit* widget = 
	new QLineEdit( theParent,
		       CreateName( "LineEdit", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
