/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgListBox.cpp,v 2.4 1999/10/16 01:50:52 jeff Exp $

*******************************************************************************/


#include "widget/DlgListBox.h"

#include "prop/ListBoxPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgListBox::itsSignals;
int DlgListBox::itsNextWidgetNum = 1;

DlgListBox::DlgListBox() :
    Inherited( DlgWidget::DW_ListBox )
{
}


DlgListBox::DlgListBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgListBox::~DlgListBox()
{
}


void DlgListBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ListBoxPage( this, parent, "ListBox" ) );

    labels.append( tr( "List Box" ) );
}


void DlgListBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "ListBox {" << endl;
    }
  
    QListBox* widget = GetListBox();

    for( unsigned int i = 0; i < widget->count(); ++i )
    {
	stream << saveIndent <<
	    "Item {" << widget->text( i ) << "}" << endl;
    }
    
    stream << saveIndent << "DragSelect {";
    SaveBool( stream, widget->dragSelect() );
    stream << "}" << endl;
  
    stream << saveIndent << "AutoScroll {";
    SaveBool( stream, widget->autoScroll() );
    stream << "}" << endl;
  
    stream << saveIndent << "ScrollBar {";
    SaveBool( stream, widget->scrollBar() );
    stream << "}" << endl;
  
    stream << saveIndent << "AutoScrollBar {";
    SaveBool( stream, widget->autoScrollBar() );
    stream << "}" << endl;
  
    stream << saveIndent << "BottomScrollBar {";
    SaveBool( stream, widget->bottomScrollBar() );
    stream << "}" << endl;
  
    stream << saveIndent << "AutoBottomScrollBar {";
    SaveBool( stream, widget->autoBottomScrollBar() );
    stream << "}" << endl;
  
    stream << saveIndent << "SmoothScrolling {";
    SaveBool( stream, widget->smoothScrolling() );
    stream << "}" << endl;

    stream << saveIndent << "MultiSelection {";
    SaveBool( stream, widget->isMultiSelection() );
    stream << "}" << endl;

    stream << saveIndent << "AutoUpdate {";
    SaveBool( stream, widget->autoUpdate() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgListBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QListBox* widget = GetListBox();

    // Compare to known keys
    if( key == "Item" )
    {
	widget->insertItem( value );
    }
    else if( key == "DragSelect" )
    {
	widget->setDragSelect( (value == "TRUE") );
    }
    else if( key == "AutoScroll" )
    {
	widget->setAutoScroll( (value == "TRUE") );
    }
    else if( key == "ScrollBar" )
    {
	widget->setScrollBar( (value == "TRUE") );
    }
    else if( key == "AutoScrollBar" )
    {
	widget->setAutoScrollBar( (value == "TRUE") );
    }
    else if( key == "BottomScrollBar" )
    {
	widget->setBottomScrollBar( (value == "TRUE") );
    }
    else if( key == "AutoBottomScrollBar" )
    {
	widget->setAutoBottomScrollBar( (value == "TRUE") );
    }
    else if( key == "SmoothScrolling" )
    {
	widget->setSmoothScrolling( (value == "TRUE") );
    }
    else if( key == "MultiSelection" )
    {
	widget->setMultiSelection( (value == "TRUE") );
    }
    else if( key == "AutoUpdate" )
    {
	widget->setAutoUpdate( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgListBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QListBox* widget = GetListBox();

    for( unsigned int i = 0; i < widget->count(); ++i )
    {
	stream << varName << "insertItem( ";
        GenerateText( widget->text( i ), widgetLayout, stream );
        stream << " );" << endl;
    }
    
    stream << varName << "setDragSelect( " <<
	(widget->dragSelect() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setAutoScroll( " <<
	(widget->autoScroll() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setScrollBar( " <<
	(widget->scrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setAutoScrollBar( " <<
	(widget->autoScrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setBottomScrollBar( " <<
	(widget->bottomScrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setAutoBottomScrollBar( " <<
	(widget->autoBottomScrollBar() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setSmoothScrolling( " <<
	(widget->smoothScrolling() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setMultiSelection( " <<
	(widget->isMultiSelection() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAutoUpdate( " <<
	(widget->autoUpdate() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgListBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgListBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "highlighted(int)" );
	itsSignals.append( "highlighted(const QString&)" );
	itsSignals.append( "highlighted(QListBoxItem *)" );
	itsSignals.append( "selected(int)" );
	itsSignals.append( "selected(const QString&)" );
	itsSignals.append( "selected(QListBoxItem *)" );
	itsSignals.append( "selectionChanged()" );
    }

    return itsSignals;
}


void DlgListBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QListBox* widget = 
	new QListBox( theParent,
		      CreateName( "ListBox", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
