/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgMultiLineEdit.cpp,v 2.3 1999/10/16 01:50:59 jeff Exp $

*******************************************************************************/


#include "widget/DlgMultiLineEdit.h"

#include "prop/MultiLineEditPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgMultiLineEdit::itsSignals;
int DlgMultiLineEdit::itsNextWidgetNum = 1;

DlgMultiLineEdit::DlgMultiLineEdit() :
    Inherited( DlgWidget::DW_MultiLineEdit )
{
}


DlgMultiLineEdit::DlgMultiLineEdit(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgMultiLineEdit::~DlgMultiLineEdit()
{
}


void DlgMultiLineEdit::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new MultiLineEditPage( this, parent, "MultiLineEdit" ) );

    labels.append( tr( "Multi Line Edit" ) );
}


void DlgMultiLineEdit::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "MultiLineEdit {" << endl;
    }
  
    QMultiLineEdit* widget = GetMultiLineEdit();

    for( int i = 0; i < widget->numLines(); ++i )
    {
	stream << saveIndent <<
	    "Line {" << widget->textLine( i ) << "}" << endl;
    }
    
    stream << saveIndent << "ReadOnly {";
    SaveBool( stream, widget->isReadOnly() );
    stream << "}" << endl;
    
    stream << saveIndent << "OverwriteMode {";
    SaveBool( stream, widget->isOverwriteMode() );
    stream << "}" << endl;

    stream << saveIndent << "AutoUpdate {";
    SaveBool( stream, widget->autoUpdate() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgMultiLineEdit::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QMultiLineEdit* widget = GetMultiLineEdit();

    // Compare to known keys
    if( key == "Line" )
    {
	widget->insertLine( value );
    } 
    else if( key == "ReadOnly" )
    {
	widget->setReadOnly( (value == "TRUE") );
    }
    else if( key == "OverwriteMode" )
    {
	widget->setOverwriteMode( (value == "TRUE") );
    }
    else if( key == "AutoUpdate" )
    {
	widget->setAutoUpdate( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgMultiLineEdit::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QMultiLineEdit* widget = GetMultiLineEdit();

    for( int i = 0; i < widget->numLines(); ++i )
    {
	stream << varName << "insertLine( \"" <<
	    widget->textLine( i ) << "\" );" << endl;
    }
    
    stream << varName << "setReadOnly( " <<
	(widget->isReadOnly() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setOverwriteMode( " <<
	(widget->isOverwriteMode() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAutoUpdate( " <<
	(widget->autoUpdate() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgMultiLineEdit::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgMultiLineEdit::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "textChanged()" );
	itsSignals.append( "returnPressed()" );
    }

    return itsSignals;
}


void DlgMultiLineEdit::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QMultiLineEdit* widget = 
	new QMultiLineEdit( theParent,
			    CreateName( "MultiLineEdit", 
					theName, 
					itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
