/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgProgressBar.cpp,v 2.3 1999/10/16 01:51:01 jeff Exp $

*******************************************************************************/


#include "widget/DlgProgressBar.h"

#include "prop/ProgressBarPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgProgressBar::itsSignals;
int DlgProgressBar::itsNextWidgetNum = 1;

DlgProgressBar::DlgProgressBar() :
    Inherited( DlgWidget::DW_ProgressBar )
{
}


DlgProgressBar::DlgProgressBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgProgressBar::~DlgProgressBar()
{
}


void DlgProgressBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ProgressBarPage( this, parent, "ProgressBar" ) );

    labels.append( tr( "Progress Bar" ) );
}


void DlgProgressBar::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "ProgressBar {" << endl;
    }
  
    QProgressBar* widget = GetProgressBar();

    stream << saveIndent <<
	"TotalSteps {" << widget->totalSteps() << "}" << endl;
    
    stream << saveIndent <<
	"Progress {" << widget->progress() << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgProgressBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QProgressBar* widget = GetProgressBar();

    // Compare to known keys
    if( key == "TotalSteps" )
    {
	widget->setTotalSteps( value.toInt() );
    } 
    else if( key == "Progress" )
    {
	widget->setProgress( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgProgressBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QProgressBar* widget = GetProgressBar();

    stream << varName << "setProgress( " << widget->progress() << " );" << endl;
}


void DlgProgressBar::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    const QProgressBar* progbar = GetProgressBar();

    stream << progbar->totalSteps() << ", " << parentName << ", " << name;
}


void DlgProgressBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgProgressBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgProgressBar::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QProgressBar* widget = 
	new QProgressBar( theParent,
		  CreateName( "ProgressBar", 
			      theName, 
			      itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
