/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgPushButton.cpp,v 2.3 1999/10/16 01:51:04 jeff Exp $

*******************************************************************************/


#include "widget/DlgPushButton.h"

#include "prop/PushButtonPage.h"
#include "prop/WidgetBitmapPage.h"

#include "GuiTools.h"

#define Inherited DlgButton

QStrList DlgPushButton::itsSignals;
int DlgPushButton::itsNextWidgetNum = 1;

DlgPushButton::DlgPushButton() :
    Inherited( DlgWidget::DW_PushButton )
{
}


DlgPushButton::DlgPushButton(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgPushButton::~DlgPushButton()
{
}


void DlgPushButton::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    DlgWidget::GetPropertyPages( parent, pages, labels );

    pages.append( new PushButtonPage( this, parent, "PushButton" ) );

    labels.append( tr( "Push Button" ) );
}


void DlgPushButton::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "PushButton {" << endl;
    }
  
    QPushButton* widget = GetPushButton();

    stream << saveIndent << "ToggleButton {";
    SaveBool( stream, widget->isToggleButton() );
    stream << "}" << endl;
    
    stream << saveIndent << "Default {";
    SaveBool( stream, widget->isDefault() );
    stream << "}" << endl;
    
    stream << saveIndent << "AutoDefault {";
    SaveBool( stream, widget->autoDefault() );
    stream << "}" << endl;

    stream << saveIndent << "IsMenuButton {";
    SaveBool( stream, widget->isMenuButton() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgPushButton::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QPushButton* widget = GetPushButton();

    // Compare to known keys
    if( key == "ToggleButton" )
    {
	widget->setToggleButton( (value == "TRUE") );
    } 
    else if( key == "Default" )
    {
	widget->setDefault( (value == "TRUE" ) );
    }
    else if( key == "AutoDefault" )
    {
	widget->setAutoDefault( (value == "TRUE") );
    }
    else if( key == "IsMenuButton" )
    {
	widget->setIsMenuButton( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgPushButton::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QPushButton* widget = GetPushButton();

    stream << varName << "setToggleButton( " << 
	(widget->isToggleButton() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setDefault( " << 
	(widget->isDefault() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setAutoDefault( " << 
	(widget->autoDefault() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setIsMenuButton( " << 
	(widget->isMenuButton() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgPushButton::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgPushButton::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgPushButton::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QPushButton* widget = 
	new QPushButton( theParent,
			 CreateName( "PushButton", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
