/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgScrollView.cpp,v 2.3 1999/10/16 01:51:10 jeff Exp $

*******************************************************************************/


#include "widget/DlgScrollView.h"

#include "prop/ScrollViewPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgScrollView::itsSignals;
int DlgScrollView::itsNextWidgetNum = 1;

DlgScrollView::DlgScrollView() :
    Inherited( DlgWidget::DW_ScrollView )
{
}


DlgScrollView::DlgScrollView(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgScrollView::~DlgScrollView()
{
}


void DlgScrollView::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ScrollViewPage( this, parent, "ScrollView" ) );

    labels.append( tr( "Scroll View" ) );
}


void DlgScrollView::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "ScrollView {" << endl;
    }
  
    QScrollView* widget = GetScrollView();

    stream << saveIndent << "ResizePolicy {" <<
	GuiTools::QScrollViewResizePolicyStr( widget->resizePolicy() ) <<
	"}" << endl;
    
    stream << saveIndent << "VScrollBarMode {" <<
	GuiTools::QScrollViewScrollBarModeStr( widget->vScrollBarMode() ) <<
	"}" << endl;
    
    stream << saveIndent << "HScrollBarMode {" <<
	GuiTools::QScrollViewScrollBarModeStr( widget->hScrollBarMode() ) <<
	"}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgScrollView::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QScrollView* widget = GetScrollView();

    // Compare to known keys
    if( key == "ResizePolicy" )
    {
	widget->setResizePolicy( GuiTools::QScrollViewResizePolicy( value ) );
    } 
    else if( key == "VScrollBarMode" )
    {
	widget->setVScrollBarMode(
	    GuiTools::QScrollViewScrollBarMode( value ) );
    }
    else if( key == "HScrollBarMode" )
    {
	widget->setHScrollBarMode(
	    GuiTools::QScrollViewScrollBarMode( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgScrollView::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QScrollView* widget = GetScrollView();

    stream << varName << "setResizePolicy( QScrollView::" <<
	GuiTools::QScrollViewResizePolicyStr( widget->resizePolicy() ) <<
	" );" << endl;
    
    stream << varName << "setVScrollBarMode( QScrollView::" <<
	GuiTools::QScrollViewScrollBarModeStr( widget->vScrollBarMode() ) <<
	" );" << endl;
    
    stream << varName << "setHScrollBarMode( QScrollView::" <<
	GuiTools::QScrollViewScrollBarModeStr( widget->hScrollBarMode() ) <<
	" );" << endl;    
}


void DlgScrollView::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgScrollView::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "contentsMoving(int,int)" );
    }

    return itsSignals;
}


void DlgScrollView::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QScrollView* widget = 
	new QScrollView( theParent,
			 CreateName( "ScrollView", 
				     theName, 
				     itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
