/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSpinBox.cpp,v 2.4 1999/10/16 01:51:15 jeff Exp $

*******************************************************************************/


#include "widget/DlgSpinBox.h"

#include "prop/SpinBoxPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgSpinBox::itsSignals;
int DlgSpinBox::itsNextWidgetNum = 1;

DlgSpinBox::DlgSpinBox() :
    Inherited( DlgWidget::DW_SpinBox )
{
}


DlgSpinBox::DlgSpinBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgSpinBox::~DlgSpinBox()
{
}


void DlgSpinBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SpinBoxPage( this, parent, "SpinBox" ) );

    labels.append( tr( "Spin Box" ) );
}


void DlgSpinBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "SpinBox {" << endl;
    }
  
    QSpinBox* widget = GetSpinBox();

    stream << saveIndent <<	
	"MinValue {" << widget->minValue() << "}" << endl;
    
    stream << saveIndent <<
	"MaxValue {" << widget->maxValue() << "}" << endl;
    
    stream << saveIndent <<
	"Step {" << widget->lineStep() << "}" << endl;
    
    stream << saveIndent <<
	"Prefix {" << widget->prefix() << "}" << endl;
    
    stream << saveIndent <<
	"Suffix {" << widget->suffix() << "}" << endl;
    
    stream << saveIndent <<
	"SpecialValue {" << widget->specialValueText() << "}" << endl;
    
    stream << saveIndent << "Wrapping {";
    SaveBool( stream, widget->wrapping() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgSpinBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QSpinBox* widget = GetSpinBox();

    // Compare to known keys
    if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    } 
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Step" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "Prefix" )
    {
	widget->setPrefix( value );
    }
    else if( key == "Suffix" )
    {
	widget->setSuffix( value );
    }
    else if( key == "SpecialValue" )
    {
	widget->setSpecialValueText( value );
    }
    else if( key == "Wrapping" )
    {
	widget->setWrapping( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgSpinBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QSpinBox* widget = GetSpinBox();

    stream << varName << "setRange( " 
	   << widget->minValue() << ", "
	   << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( "
	   << widget->lineStep() << ", "
	   << "0" << " );" << endl;
    
    stream << varName << "setPrefix( ";
    GenerateText(  widget->prefix(), widgetLayout, stream );
    stream << " );" << endl;

    stream << varName << "setSuffix( ";
    GenerateText(  widget->suffix(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setSpecialValueText( ";
    GenerateText( widget->specialValueText(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setWrapping( "
	   << (widget->wrapping() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgSpinBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSpinBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "valueChanged(const QString&)" );
    }

    return itsSignals;
}


void DlgSpinBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QSpinBox* widget = 
	new QSpinBox( theParent,
		      CreateName( "SpinBox", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
