/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSplitter.cpp,v 2.5 1999/10/16 01:51:17 jeff Exp $

*******************************************************************************/


#include "widget/DlgSplitter.h"

#include "prop/SplitterPage.h"

#include "GuiTools.h"

#define Inherited DlgFrame

QStrList DlgSplitter::itsSignals;
int DlgSplitter::itsNextWidgetNum = 1;

DlgSplitter::DlgSplitter() :
    Inherited( DlgWidget::DW_Splitter ),
    mySpacing( 5 )
{
}


DlgSplitter::DlgSplitter(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    mySpacing( 5 )
{
}


DlgSplitter::~DlgSplitter()
{
}


void DlgSplitter::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SplitterPage( this, parent, "Splitter" ) );

    labels.append( tr( "Splitter" ) );
}


void DlgSplitter::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "Splitter {" << endl;
    }

    QSplitter *splitter = GetSplitter();
    Qt::Orientation o = splitter->orientation();
    stream << saveIndent << "Orientation {";
    if( o == Qt::Horizontal )
      stream << "Horizontal";
    else
      stream << "Vertical";
    stream << "}" << endl;
  
    stream << saveIndent << "OpaqueResize {";
    SaveBool( stream, splitter->opaqueResize() );
    stream << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgSplitter::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    QSplitter *splitter = GetSplitter();

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Orientation" )
    {
      Qt::Orientation o;
      if( value == "Horizontal" )
	o = Qt::Horizontal;
      else
	o = Qt::Vertical;
      splitter->setOrientation( o );
    } 
    else if( key == "OpaqueResize" )
    {
      splitter->setOpaqueResize( value == "TRUE" );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgSplitter::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    QSplitter *splitter = GetSplitter();
    Qt::Orientation o = splitter->orientation();

    stream << varName << "setOrientation( ";
    if( o == Qt::Horizontal )
      stream << "Horizontal";
    else
      stream << "Vertical";
    stream << " );" << endl;

    stream << varName << "setOpaqueResize( " <<
	(splitter->opaqueResize() ? "TRUE" : "FALSE") << " );" << endl;

}


void DlgSplitter::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSplitter::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgSplitter::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QSplitter* widget = 
	new QSplitter( theParent,
                   CreateName( "Splitter", 
                               theName, 
                               itsNextWidgetNum ) );
    
    SetQtWidget( widget, theParent );
}
