/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgTabBar.cpp,v 2.3 1999/10/16 01:51:19 jeff Exp $

*******************************************************************************/


#include "widget/DlgTabBar.h"

#include "prop/TabBarPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgTabBar::itsSignals;
int DlgTabBar::itsNextWidgetNum = 1;

DlgTabBar::DlgTabBar() :
    Inherited( DlgWidget::DW_TabBar )
{
    myLabelList.setAutoDelete( TRUE );
}


DlgTabBar::DlgTabBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
    myLabelList.setAutoDelete( TRUE );
}


DlgTabBar::~DlgTabBar()
{
}


void DlgTabBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new TabBarPage( this, parent, "TabBar" ) );

    labels.append( tr( "Tab Bar" ) );
}


void DlgTabBar::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "TabBar {" << endl;
    }
  
    QTabBar* widget = GetTabBar();

    QStrListIterator iter( myLabelList );
    const char* label;
    while( (label = iter()) )
    {
	stream << saveIndent << "Tab {" << label << "}" << endl;
    }

    stream << saveIndent << "Shape {" <<
	GuiTools::QTabBarShapeStr( widget->shape() ) << "}" << endl;

    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgTabBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QTabBar* widget = GetTabBar();

    // Compare to known keys
    if( key == "Tab" )
    {
	myLabelList.append( value );
    }
    else if( key == "Shape" )
    {
	widget->setShape( GuiTools::QTabBarShape( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}


void DlgTabBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QTabBar* widget = GetTabBar();

    stream << varName << "setShape( QTabBar::" << 
	GuiTools::QTabBarShapeStr( widget->shape() ) << " );\n";

    stream << srcIndent << "{" << endl;
    bool first = TRUE;
    QStrListIterator iter( myLabelList );
    const char* label;
    while( (label = iter()) )
    {
	if( first )
	{
	    stream << srcIndent << srcIndent << "QTab* tab = 0;" << endl;
	    first = FALSE;
	}
	
	stream << srcIndent << srcIndent << "tab = new QTab;" << endl

               << srcIndent << srcIndent 
               << "tab->label = ";
        GenerateText( label, widgetLayout, stream );
        stream << ";" << endl

               << srcIndent << varName << "addTab( tab );" << endl

               << endl;
    }
    stream << srcIndent << "}" << endl;
}


void DlgTabBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgTabBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "selected(int)" );
    }

    return itsSignals;
}


void DlgTabBar::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QTabBar* widget = 
	new QTabBar( theParent,
		     CreateName( "TabBar", 
				 theName, 
				 itsNextWidgetNum ) );

    QTab* tab = new QTab;
    tab->label = tr( "Tab 1" );
    widget->addTab( tab );

    tab = new QTab;
    tab->label = tr( "Tab 2" );
    widget->addTab( tab );

    tab = new QTab;
    tab->label = tr( "Tab 3" );
    widget->addTab( tab );

    SetQtWidget( widget, theParent );
}
