/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgTextBrowser.cpp,v 2.4 1999/10/16 01:51:21 jeff Exp $

*******************************************************************************/


#include "widget/DlgTextBrowser.h"

#include "prop/TextBrowserPage.h"

#include "GuiTools.h"

#define Inherited DlgWidget

QStrList DlgTextBrowser::itsSignals;
int DlgTextBrowser::itsNextWidgetNum = 1;

DlgTextBrowser::DlgTextBrowser() :
    Inherited( DlgWidget::DW_TextBrowser )
{
  useSource = false;
}


DlgTextBrowser::DlgTextBrowser(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
  useSource = false;
}


DlgTextBrowser::~DlgTextBrowser()
{
}


void DlgTextBrowser::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new TextBrowserPage( this, parent, "TextBrowser" ) );

    labels.append( tr( "Text Browser" ) );
}


void DlgTextBrowser::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
  if( topLevel )
  {
    stream << "TextBrowser {" << endl;
  }
  
  QTextBrowser* widget = GetTextBrowser();

  if( useSource )
  {
    stream << saveIndent <<
      "Source {" << itsSource << "}" << endl;
  } else
  {
    stream << saveIndent <<
      "Text {" << widget->text() << "}" << endl;
  }
    
  Inherited::SaveContents( stream, FALSE );

  if( topLevel )
  {
    stream << "}" << endl;
  }
}


bool DlgTextBrowser::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
  bool retVal = TRUE;

  if( topLevel )
  {
    if( key == "Name" )
    {
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
    }
  }

  QTextBrowser* widget = GetTextBrowser();

  // Compare to known keys
  if( key == "Text" )
  {
    widget->setText( value );
    useSource = false;
  } 
  else if( key == "Source" )
  {
    itsSource = value;
    useSource = true;
  }
  else
  {
    retVal = Inherited::RestoreKeyValue( key, value, FALSE );
  }

  return retVal;
}


void DlgTextBrowser::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QTextBrowser* widget = GetTextBrowser();

    if( useSource )
    {
        stream << varName << 
            "setSource( \"" << itsSource << "\" );" << endl;
    } else
    {
        stream << varName << "setText( ";
        GenerateText( widget->text(), widgetLayout, stream );
        stream << " );" << endl;
    }
}


void DlgTextBrowser::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgTextBrowser::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
      itsSignals.append( "backwardAvailable(bool)" );
      itsSignals.append( "forwardAvailable(bool)" );
      itsSignals.append( "highlighted(const QString&)" );
      itsSignals.append( "textChanged()" );
    }

    return itsSignals;
}


void DlgTextBrowser::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QTextBrowser* widget = 
	new QTextBrowser( theParent,
		       CreateName( "TextBrowser", 
				   theName, 
				   itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
