/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgVBox.cpp,v 2.4 1999/10/16 01:51:28 jeff Exp $

*******************************************************************************/


#include "widget/DlgVBox.h"

#include "GuiTools.h"

#define Inherited DlgHBox

QStrList DlgVBox::itsSignals;
int DlgVBox::itsNextWidgetNum = 1;

DlgVBox::DlgVBox() :
    Inherited( DlgWidget::DW_VBox )
{
}


DlgVBox::DlgVBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgVBox::~DlgVBox()
{
}


void DlgVBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );
}


void DlgVBox::SaveContents(
    QTextStream&	stream,
    bool		topLevel
    )
{
    if( topLevel )
    {
	stream << "VBox {" << endl;
    }
  
    Inherited::SaveContents( stream, FALSE );

    if( topLevel )
    {
	stream << "}" << endl;
    }
}


bool DlgVBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    retVal = Inherited::RestoreKeyValue( key, value, FALSE );

    return retVal;
}


void DlgVBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );
}


void DlgVBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgVBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgVBox::CreateQtWidget(
    QWidget*	theParent,
    const char*	theName
    )
{
    QVBox* widget = 
	new QVBox( theParent,
                   CreateName( "VBox", 
                               theName, 
                               itsNextWidgetNum ) );

    SetQtWidget( widget, theParent );
}
