/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgWidget.cpp,v 2.9 1999/10/16 01:51:32 jeff Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qpalette.h>
#include <qregexp.h>

#include "GuiTools.h"
#include "WidgetLayout.h"

#include "widget/DlgWidget.h"
#include "widget/DlgButton.h"
#include "widget/DlgButtonGroup.h"
#include "widget/DlgCheckBox.h"
#include "widget/DlgComboBox.h"
#include "widget/DlgFrame.h"
#include "widget/DlgGrid.h"
#include "widget/DlgGroupBox.h"
#include "widget/DlgHBox.h"
#include "widget/DlgHButtonGroup.h"
#include "widget/DlgLCDNumber.h"
#include "widget/DlgLabel.h"
#include "widget/DlgLineEdit.h"
#include "widget/DlgListBox.h"
#include "widget/DlgListView.h"
#include "widget/DlgMenuBar.h"
#include "widget/DlgMultiLineEdit.h"
#include "widget/DlgProgressBar.h"
#include "widget/DlgPushButton.h"
#include "widget/DlgRadioButton.h"
#include "widget/DlgScrollBar.h"
#include "widget/DlgScrollView.h"
#include "widget/DlgSlider.h"
#include "widget/DlgSpinBox.h"
#include "widget/DlgSplitter.h"
#include "widget/DlgTabBar.h"
#include "widget/DlgTextView.h"
#include "widget/DlgTextBrowser.h"
#include "widget/DlgUser.h"
#include "widget/DlgVBox.h"
#include "widget/DlgVButtonGroup.h"
#include "pixmap/PixmapDraw.h"

#include "prop/WidgetBitmapPage.h"
#include "prop/WidgetPropDialog.h"

#include "bmp/PixmapFactory.h"


QString DlgWidget::saveIndent = "\t\t";
QString DlgWidget::srcIndent = "    ";


DlgWidget::DlgWidget(
    Type	type
    ) :
    useBackgroundPixmap( FALSE ),
    itsWidget( 0 ),
    itsType( type ),
    isInLayout( FALSE ),
    itsOriginalParent( 0 ),
    adjustSize( FALSE ),
    addPixmapToProject( FALSE ),
    readPixmapFromData( FALSE),
    isEnabled( TRUE ),
    itsParentForSource(NULL)
{
    itsConnectedSignalList.setAutoDelete( TRUE );
}


DlgWidget::~DlgWidget()
{
    itsConnectedSignalList.clear();
}


DlgWidget *DlgWidget::NewDlgWidget( 
    QWidget *parent,
    DlgWidget::Type type,
    const QString *name 
    )
{
    DlgWidget *newWidget = NULL;

    const char *theName = NULL;

    if( name )
	theName = *name;

#define NEW_DLG_WIDGET_BY_TYPE(type) \
    case DlgWidget::DW_##type:				\
    {							\
	newWidget = new Dlg##type;	\
	break;						\
    }
    
    switch( type )
    {
	NEW_DLG_WIDGET_BY_TYPE( Button )
	NEW_DLG_WIDGET_BY_TYPE( ButtonGroup )
	NEW_DLG_WIDGET_BY_TYPE( CheckBox )
	NEW_DLG_WIDGET_BY_TYPE( ComboBox )
	NEW_DLG_WIDGET_BY_TYPE( Frame )
	NEW_DLG_WIDGET_BY_TYPE( Grid )
	NEW_DLG_WIDGET_BY_TYPE( GroupBox )
	NEW_DLG_WIDGET_BY_TYPE( HBox )
	NEW_DLG_WIDGET_BY_TYPE( HButtonGroup )
	NEW_DLG_WIDGET_BY_TYPE( LCDNumber )
	NEW_DLG_WIDGET_BY_TYPE( Label )
	NEW_DLG_WIDGET_BY_TYPE( LineEdit )
	NEW_DLG_WIDGET_BY_TYPE( ListBox )
	NEW_DLG_WIDGET_BY_TYPE( ListView )
	NEW_DLG_WIDGET_BY_TYPE( MenuBar )
	NEW_DLG_WIDGET_BY_TYPE( MultiLineEdit )
	NEW_DLG_WIDGET_BY_TYPE( ProgressBar )
	NEW_DLG_WIDGET_BY_TYPE( PushButton )
	NEW_DLG_WIDGET_BY_TYPE( RadioButton )
	NEW_DLG_WIDGET_BY_TYPE( ScrollBar )
	NEW_DLG_WIDGET_BY_TYPE( ScrollView )
	NEW_DLG_WIDGET_BY_TYPE( Slider )
	NEW_DLG_WIDGET_BY_TYPE( SpinBox )
	NEW_DLG_WIDGET_BY_TYPE( Splitter )
	NEW_DLG_WIDGET_BY_TYPE( TabBar )
	NEW_DLG_WIDGET_BY_TYPE( TextView )
	NEW_DLG_WIDGET_BY_TYPE( TextBrowser )
	NEW_DLG_WIDGET_BY_TYPE( User )
	NEW_DLG_WIDGET_BY_TYPE( VBox )
	NEW_DLG_WIDGET_BY_TYPE( VButtonGroup )

	case DlgWidget::DW_None:
	{
	    break;
	}
    }

    newWidget->CreateQtWidget( parent, theName );

    return newWidget;
}


DlgWidget *DlgWidget::NewDlgWidget( 
    QWidget *parent, 
    const QString &type, 
    const QString *name 
    )
{
    DlgWidget *newWidget = NULL;

    const char *theName = NULL;

    if( name )
	theName = *name;

#define ELSE_IF_TYPE( x )		\
    else if( type == #x )		\
	newWidget = new Dlg##x;	

    if( type == "Button" )
	newWidget = new DlgButton;
    ELSE_IF_TYPE( ButtonGroup )
    ELSE_IF_TYPE( CheckBox )
    ELSE_IF_TYPE( ComboBox )
    ELSE_IF_TYPE( Frame )
    ELSE_IF_TYPE( Grid )
    ELSE_IF_TYPE( GroupBox )
    ELSE_IF_TYPE( HBox )
    ELSE_IF_TYPE( HButtonGroup )
    ELSE_IF_TYPE( LCDNumber )
    ELSE_IF_TYPE( Label )
    ELSE_IF_TYPE( LineEdit )
    ELSE_IF_TYPE( ListBox )
    ELSE_IF_TYPE( ListView )
    ELSE_IF_TYPE( MenuBar )
    ELSE_IF_TYPE( MultiLineEdit )
    ELSE_IF_TYPE( ProgressBar )
    ELSE_IF_TYPE( PushButton )
    ELSE_IF_TYPE( RadioButton )
    ELSE_IF_TYPE( ScrollBar )
    ELSE_IF_TYPE( ScrollView )
    ELSE_IF_TYPE( Slider )
    ELSE_IF_TYPE( SpinBox )
    ELSE_IF_TYPE( Splitter )
    ELSE_IF_TYPE( TabBar )
    ELSE_IF_TYPE( TextView )
    ELSE_IF_TYPE( TextBrowser )
    ELSE_IF_TYPE( User )
    ELSE_IF_TYPE( VBox )
    ELSE_IF_TYPE( VButtonGroup )

    newWidget->CreateQtWidget( parent, theName );

    return newWidget;
}


QPoint DlgWidget::GetPos() const
{
    QWidget* widget = GetWidget();
    return itsOriginalParent->mapFromGlobal(widget->mapToGlobal(QPoint(0,0)));
}


int DlgWidget::DoPropertyDialog()
{
    WidgetPropDialog dlg( this, NULL /*GetWidget()*/, "Properties" );
    
    int rc = dlg.exec();
    
    GetWidget()->update();
    
    return rc;
}


void DlgWidget::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    pages.append( new WidgetBitmapPage( this, parent, "Bitmap" ) );

    labels.append( tr( "Bitmap" ) );
}


QString DlgWidget::GetGeneratedVarName() const
{
    QString str = "qtarch_";
    
    if( IsVariableMapped() )
    {
	str = GetVariableName();
    }
    else
    {
	str += GetWidgetName();
    }

    return str;
}


void DlgWidget::SetInLayout(
    QWidget*	newParent 
    )
{
    isInLayout = TRUE;

    QWidget::FocusPolicy focusPolicy = GetWidget()->focusPolicy();
    
    // If a new parent is given, set the widget's parent to it.  If not given,
    // set the parent to the widget's original parent.
    if( newParent )
    {
	QWidget* widget = GetWidget();
	QPoint pos = newParent->mapFromGlobal(
	    widget->mapToGlobal( QPoint(0,0) ) );
	
	widget->recreate( newParent, 0, pos, TRUE );
    }
    else
    {
	// Only set if the widget's parent is currently not the original 
	// parent.
	if( GetWidget()->parent() != itsOriginalParent )
	{
	    QWidget* widget = GetWidget();
	    QPoint pos = itsOriginalParent->mapFromGlobal( 
		widget->mapToGlobal( QPoint(0,0) ) );
		
	    widget->recreate( itsOriginalParent, 0, pos, TRUE );
	}
    }

    GetWidget()->setFocusPolicy( focusPolicy );

    emit LayoutStatusChanged( this );
}


void DlgWidget::RemoveFromLayout()
{
    isInLayout = FALSE;

    // Don't recreate if the app is going down
    if( !QApplication::closingDown() )
    {
	QWidget* widget = GetWidget();

	QWidget::FocusPolicy focusPolicy = widget->focusPolicy();

	QPoint pos = 
	    itsOriginalParent->mapFromGlobal(widget->mapToGlobal(QPoint(0,0)));
    
	widget->recreate( itsOriginalParent, 0, pos, TRUE );

	widget->setMinimumSize( 0,0 );
	widget->setMaximumSize( QWIDGETSIZE_MAX,QWIDGETSIZE_MAX );
	widget->setFocusPolicy( focusPolicy );

	emit LayoutStatusChanged( this );
    }
}


void DlgWidget::SaveContents( 
    QTextStream&	stream,
    bool		/*topLevel*/
    )
{
    stream << saveIndent <<
	"Rect {" << MapGeometryToOriginalParent( itsWidget ) << "}" << endl;

    stream << saveIndent << 
	"Name {" << itsWidget->name() << "}" << endl;

    if( IsVariableMapped() )
    {
	stream << saveIndent << 
	    "Variable {" << itsVariableName << "}" << endl;
    }

    // user changed palette
    const QPalette palette = itsWidget->palette();
    if( palette != QApplication::palette() )
    {
        stream << saveIndent << 
	    "Palette {" << palette << "}" << endl;
    }

    // user changed font
    const QFont font = itsWidget->font();
    if( font != QApplication::font() )
    {
	stream << saveIndent <<
	    "Font {" << font << "}" << endl;
    }

    QListIterator<SignalConnection> iter( itsConnectedSignalList );
    SignalConnection* sig;
    
    while( (sig = iter()) )
    {
	stream << saveIndent <<
	    "Signal {" << sig->ToString() << "}" << endl;
    }

    stream << saveIndent <<
	"MinimumSize {" << itsWidget->minimumSize() << "}" << endl;
    stream << saveIndent <<
	"MaximumSize {" << itsWidget->maximumSize() << "}" << endl;

    stream << saveIndent <<
	"BackgroundPixmap {" << backgroundPixmapsName << "}" << endl;

    stream << saveIndent <<
	"UseBackgroundPixmap {"; 
    SaveBool( stream, useBackgroundPixmap ); 
    stream << "}" << endl;

    stream << saveIndent <<
	"AdjustSize {"; 
    SaveBool( stream, adjustSize ); 
    stream << "}" << endl;

    stream << saveIndent <<
	"ReadPixmapFromData {"; 
    SaveBool( stream, readPixmapFromData ); 
    stream << "}" << endl;

    stream << saveIndent <<
	"DataVarName {" << dataVarName << "}" << endl;
    stream << saveIndent <<
	"DataLenName {" << dataLenName << "}" << endl;

    stream << saveIndent <<
	"FocusPolicy {" << 
	GuiTools::QWidgetFocusPolicyStr( itsWidget->focusPolicy() ) << 
	"}" << endl;

    stream << saveIndent <<
	"BackgroundMode {" <<
	GuiTools::QWidgetBackgroundModeStr( itsWidget->backgroundMode() ) << 
	"}" << endl;

    stream << saveIndent <<
	"FontPropagation {" <<
	GuiTools::QWidgetPropagationModeStr( itsWidget->fontPropagation() ) <<
	"}" << endl;
    
    stream << saveIndent <<
	"PalettePropagation {" <<
	GuiTools::QWidgetPropagationModeStr( itsWidget->palettePropagation() ) <<
	"}" << endl;
}


bool DlgWidget::RestoreKeyValue( 
    const QString&	key, 
    const QString&	value,
    bool		/*topLevel*/
    )
{
    bool rc = TRUE;

    QString val = value;
    QTextStream read( val, IO_ReadOnly );

    if( key == "Name" )
    {
	itsWidget->setName( value );
    } 
    else if( key == "Rect" )
    {
	QRect aRect;
	read >> aRect;
	itsWidget->setGeometry( aRect );
    } 
    else if( key == "Variable" )
    {
	itsVariableName = value;
    }
    else if( key == "Palette" )
    {
	QPalette palette;
	read >> palette;
	itsWidget->setPalette( palette );
    }
    else if( key == "Font" )
    {
	QFont font;
	read >> font;
	itsWidget->setFont( font );
    }
    else if( key == "Signal" )
    {
	SignalConnection* sig = new SignalConnection;
	sig->FromString( value );
	
	itsConnectedSignalList.append( sig );
    }
    else if( key == "MinimumSize" )
    {
	QSize size;
	read >> size;
	itsWidget->setMinimumSize( size );
    }
    else if( key == "MaximumSize" )
    {
	QSize size;
	read >> size;
	itsWidget->setMaximumSize( size );
    }
    else if( key == "BackgroundPixmap" )
    {
	backgroundPixmapsName = value;
    }
    else if( key == "UseBackgroundPixmap" )
    {
	RestoreBool( read, useBackgroundPixmap );
	if( ! backgroundPixmapsName.isEmpty() && useBackgroundPixmap )
	{
	    QPixmap pixmap( backgroundPixmapsName );
	    if( itsWidget->inherits( "QButton" ) )
		((QButton *)itsWidget)->setPixmap( pixmap );
	    else
		itsWidget->setBackgroundPixmap( pixmap );
	}  
    }
    else if( key == "AdjustSize" )
    {
	RestoreBool( read, adjustSize );
	if( adjustSize && useBackgroundPixmap )
	{
	    const QPixmap *pixmap = itsWidget->backgroundPixmap();
	    if( pixmap )
		itsWidget->setFixedSize( pixmap->size() );
	}
    }
    else if( key == "ReadPixmapFromData" )
    {
	RestoreBool( read, readPixmapFromData );
    }
    else if( key == "DataVarName" )
    {
	dataVarName = value;
    }
    else if( key == "DataLenName" )
    {
	dataLenName = value;
    }
    else if( key == "FocusPolicy" )
    {
	itsWidget->setFocusPolicy( GuiTools::QWidgetFocusPolicy( value ) );
    }
    else if( key == "BackgroundMode" )
    {
	QWidget::BackgroundMode mode = GuiTools::QWidgetBackgroundMode( value );
	if( (mode != QWidget::FixedPixmap) && (mode != QWidget::FixedColor) )
	{
	    itsWidget->setBackgroundMode( mode );
	}
    }
    else if( key == "FontPropagation" )
    {
	itsWidget->setFontPropagation( 
	    GuiTools::QWidgetPropagationMode( value ) );
    }
    else if( key == "PalettePropagation" )
    {
	itsWidget->setPalettePropagation(
	    GuiTools::QWidgetPropagationMode( value ) );
    }
    else if( key == "Enabled" )
    {
        isEnabled = (value == "TRUE");
    }
    else
    {
	// Key unknown
	rc = FALSE;
    }
    
    return rc;
}


void DlgWidget::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << parentName << ", " << name;
}


void DlgWidget::GenerateSource(
    QTextStream&	stream,
    const QString&	varName,
    const WidgetLayout& /*widgetLayout*/
    )
{
    stream << varName << "setGeometry(";
    if(itsParentForSource!=NULL)
        stream << itsWidget->x()-itsParentForSource->GetWidget()->x() << ", " << itsWidget->y()-itsParentForSource->GetWidget()->y();
    else
        stream << itsWidget->x() << ", " << itsWidget->y();
    stream << ", " << itsWidget->width() << ", " << itsWidget->height() << ");\n";

    stream << varName << "setMinimumSize(" << itsWidget->minimumSize().width() << ", " <<  itsWidget->minimumSize().height() << ");\n"
           << varName << "setMaximumSize(" << itsWidget->maximumSize().width() << ", " << itsWidget->maximumSize().height() << ");\n";

    const QPalette palette = itsWidget->palette();
    if( palette != QApplication::palette() )
    {
	stream << srcIndent<< "{\n"

               << srcIndent << srcIndent <<"QColorGroup normal;" << endl;
        stream << srcIndent << srcIndent 
               << "normal.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.normal().foreground().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent 
               << "normal.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.normal().button().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.normal().light().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.normal().midlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.normal().dark().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.normal().mid().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.normal().text().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.normal().brightText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.normal().buttonText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.normal().base().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.normal().background().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.normal().shadow().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.normal().highlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "normal.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.normal().highlightedText().rgb() << " ) ) );" << endl;

        stream << srcIndent << srcIndent << "QColorGroup disabled;" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.disabled().foreground().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.disabled().button().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.disabled().light().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.disabled().midlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.disabled().dark().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.disabled().mid().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.disabled().text().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.disabled().brightText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.disabled().buttonText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.disabled().base().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.disabled().background().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.disabled().shadow().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.disabled().highlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "disabled.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.disabled().highlightedText().rgb() << " ) ) );" << endl;

        stream << srcIndent << srcIndent << "QColorGroup active;" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Foreground, "
               << "QColor( QRgb( " 
               << palette.active().foreground().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Button, "
               << "QColor( QRgb( " 
               << palette.active().button().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Light, "
               << "QColor( QRgb( " 
               << palette.active().light().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Midlight, "
               << "QColor( QRgb( " 
               << palette.active().midlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Dark, "
               << "QColor( QRgb( " 
               << palette.active().dark().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Mid, "
               << "QColor( QRgb( " 
               << palette.active().mid().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Text, "
               << "QColor( QRgb( " 
               << palette.active().text().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::BrightText, "
               << "QColor( QRgb( " 
               << palette.active().brightText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::ButtonText, "
               << "QColor( QRgb( " 
               << palette.active().buttonText().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Base, "
               << "QColor( QRgb( " 
               << palette.active().base().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Background, "
               << "QColor( QRgb( " 
               << palette.active().background().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Shadow, "
               << "QColor( QRgb( " 
               << palette.active().shadow().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::Highlight, "
               << "QColor( QRgb( " 
               << palette.active().highlight().rgb() << " ) ) );" << endl;
        stream << srcIndent << srcIndent
               << "active.setColor( QColorGroup::HighlightedText, "
               << "QColor( QRgb( " 
               << palette.active().highlightedText().rgb() << " ) ) );" << endl;

	stream << srcIndent << srcIndent
               << "QPalette palette( normal, disabled, active );\n";
	stream << srcIndent << varName << "setPalette( palette );\n";
	stream << srcIndent << "}\n";
    }

    const QFont font = itsWidget->font();
    if( font != QApplication::font() )
    {
	stream << srcIndent << "{\n"
	       << srcIndent << srcIndent 
               << "QFont font(\"" << font.family() << "\", " 
               << font.pointSize() << ", "
	       << font.weight() << ", " 
               << font.italic() << ");\n";
	if( font.underline() )
	{
	    stream << srcIndent << srcIndent << "font.setUnderline(TRUE);\n";
	}
	if( font.strikeOut() )
	{
	    stream << srcIndent << srcIndent << "font.setStrikeOut(TRUE);\n";
	}
	if( font.fixedPitch() )
	{
	    stream << srcIndent << srcIndent << "font.setFixedPitch(TRUE);\n";
	}
	stream << srcIndent << srcIndent 
               << "font.setStyleHint((QFont::StyleHint)" << font.styleHint() 
               << ");\n"
	       << srcIndent << srcIndent 
               << "font.setCharSet((QFont::CharSet)" << font.charSet()
               << ");\n"
	       << srcIndent << varName << "setFont( font );\n"
	       << srcIndent << "}\n";
    }

    // Generate the background image code
    if( useBackgroundPixmap )
    {
	QString pixName = "pixmap_" + varName;
	stream << endl 
               << srcIndent << '{' << endl 
               << srcIndent << srcIndent << "QPixmap pixmap;\n";
	if( readPixmapFromData )
	{
	    if( addPixmapToProject )
	    {
		// We could expect the data somewhere in the binary
		stream << srcIndent << srcIndent 
                       << "pixmap.loadFromData(" << dataVarName << " , " 
                       << dataLenName << ");\n";
	    } else
	    {
		// Create the pixmap data right here and use it.
		QPixmap pixmap( backgroundPixmapsName );
		PixmapDraw::GeneratePixmap( &pixmap, dataVarName, dataLenName, (QFile *) stream.device() );
		stream << srcIndent << srcIndent 
                       << "pixmap.loadFromData(" << dataVarName << " , "
                       << dataLenName << ");\n";
	    }
	} else
	{
	    stream << srcIndent << srcIndent 
                   << "pixmap.load(\"" << backgroundPixmapsName << "\");\n";
	}
	if( itsWidget->inherits( "QButton" ) )
	    stream << srcIndent << varName << "setPixmap(pixmap);\n" 
                   << srcIndent << "}\n";
	else
	    stream << srcIndent << varName << "setBackgroundPixmap(pixmap);\n"
                   << srcIndent << "}\n";
    }

    stream << varName << "setFocusPolicy(QWidget::" 
           << GuiTools::QWidgetFocusPolicyStr(itsWidget->focusPolicy() ) 
           << ");\n";

    QWidget::BackgroundMode mode = itsWidget->backgroundMode();
    if( (mode != QWidget::FixedPixmap) && (mode != QWidget::FixedColor) )
    {
	stream << varName << "setBackgroundMode(QWidget::" 
               << GuiTools::QWidgetBackgroundModeStr( mode ) << ");\n";
    }

    stream << varName << "setFontPropagation(QWidget::" 
           << GuiTools::QWidgetPropagationModeStr(itsWidget->fontPropagation())
           << ");\n"
           << varName << "setPalettePropagation(QWidget::" 
           << GuiTools::QWidgetPropagationModeStr(itsWidget->palettePropagation()) 
           << ");\n";
}


void DlgWidget::GenerateSourceConnects(
    QTextStream& stream, 
    const QString& varName
    )
{
    QListIterator<SignalConnection> iter( GetConnectedSignalList() );
    SignalConnection* sig;
    QString dot = Dot();
 
    // remove the initial tab and the . or -> from the variable name to use
    // in the connect statement
    QString name = varName.right( varName.length() - srcIndent.length() );
    name = name.left( name.length() - dot.length() );

    while( (sig = iter()) )
    {
        stream << srcIndent 
               << "connect(" << name 
               << ", SIGNAL(" << sig->GetSignal() 
               << "), SLOT(" << sig->GetSlot() << "));\n";
    }
}


void DlgWidget::GenerateText(
    const QString&      text,
    const WidgetLayout& widgetLayout,
    QTextStream&        stream
    )
{
    if( !text.isEmpty() && widgetLayout.IsTextTranslated() )
    {
        stream << "tr( \"" << text << "\" )";
    }
    else
    {
        stream << "\"" << text << "\"";
    }
}


QString DlgWidget::CreateName( 
    QString className, 
    const char *name, 
    int &number 
    )
{
    QString tmp;

    if( ! name )
    {
	tmp.setNum( number );
	tmp = className + QString( "_" ) + tmp;
	number++;
    } 
    else
    {
	tmp = name;
    }

    return tmp;
}


void DlgWidget::UpdateNextWidgetNumber( 
    const QString&	widgetName,
    int&		nextWidgetNum 
    )
{
    int pos = widgetName.findRev( '_' );

    if( pos >= 0 )
    {
	QString numStr = widgetName.mid( pos + 1, 
					 widgetName.length() - pos - 1 );

	int num = numStr.toInt();
	if( num >= nextWidgetNum )
	{
	    nextWidgetNum = num + 1;
	}
    }    
}


const char* DlgWidget::Dot() const
{
    // all variables are now dynamically created
    //return IsVariableMapped() ? "." : "->";
    return "->";
}


void DlgWidget::GetSignalList( 
    QStrList& signalList
    ) const
{
    signalList.clear();
    
    AddSignalsToList( signalList );
}


void DlgWidget::AddSignalsToList( 
    QStrList&	// signalList
    ) const 
{
}


void DlgWidget::SetBackgroundData( 
    bool use, 
    bool adjust, 
    bool add, 
    bool read, 
    const QString &varName, 
    const QString &lenName, 
    const QString &fileName 
    )
{
    backgroundPixmapsName = fileName;
    
    useBackgroundPixmap = use;
    adjustSize = adjust;
    addPixmapToProject = add;
    readPixmapFromData = read;
    dataVarName = varName;
    dataLenName = lenName;
}


void DlgWidget::GetBackgroundData( 
    bool &use, 
    bool &adjust, 
    bool &add,
    bool &read, 
    QString &varName, 
    QString &lenName, 
    QString &fileName 
    )
{
    fileName = backgroundPixmapsName;

    use = useBackgroundPixmap;
    adjust = adjustSize;
    add = addPixmapToProject;
    read = readPixmapFromData;
    varName = dataVarName;
    lenName = dataLenName;
}


const QPixmap& DlgWidget::GetPixmap( DlgWidget::Type type )
{
    BitmapType bitmapType = Frame_bmp;

#define DW_PIXMAP( widget ) \
    case DW_##widget: bitmapType = widget##_bmp; break;
    
    // Translate widget type to bitmap type
    switch( type )
    {
	DW_PIXMAP( Button )
	DW_PIXMAP( ButtonGroup )
	DW_PIXMAP( CheckBox )
	DW_PIXMAP( ComboBox )
	DW_PIXMAP( Frame )
	DW_PIXMAP( Grid )
	DW_PIXMAP( GroupBox )
	DW_PIXMAP( HBox )
	DW_PIXMAP( HButtonGroup )
	DW_PIXMAP( LCDNumber )
	DW_PIXMAP( Label )
	DW_PIXMAP( LineEdit )
	DW_PIXMAP( ListBox )
	DW_PIXMAP( ListView )
	DW_PIXMAP( MenuBar )
	DW_PIXMAP( MultiLineEdit )
	DW_PIXMAP( ProgressBar )
	DW_PIXMAP( PushButton )
	DW_PIXMAP( RadioButton )
	DW_PIXMAP( ScrollBar )
	DW_PIXMAP( ScrollView )
	DW_PIXMAP( Slider )
	DW_PIXMAP( SpinBox )
	DW_PIXMAP( Splitter )
	DW_PIXMAP( TabBar )
	DW_PIXMAP( TextBrowser )
	DW_PIXMAP( TextView )
	DW_PIXMAP( User )
	DW_PIXMAP( VBox )
	DW_PIXMAP( VButtonGroup )

	case DW_None:
	default:
	{
	    break;
	}
    }
    
    return PixmapFactory::GetPixmap( bitmapType );
}


QRect DlgWidget::MapGeometryToOriginalParent(
    const QWidget*	theWidget
    ) const
{
    QRect rect = theWidget->rect();
    
    return QRect( 
	itsOriginalParent->mapFromGlobal( 
	    theWidget->mapToGlobal( rect.topLeft() ) ),
	itsOriginalParent->mapFromGlobal( 
	    theWidget->mapToGlobal( rect.bottomRight() ) ) );
}


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QSize&	size 
    )
{
    ostream << size.width() << ' ' << size.height();
    
    return ostream;
}


QTextStream& operator >>( 
    QTextStream&	istream,
    QSize&		size 
    )
{
    istream >> size.rwidth() >> size.rheight();
    
    return istream;
}


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QPoint&	point 
    )
{
    ostream << point.x() << ' ' << point.y();
    
    return ostream;
}


QTextStream& operator >>( 
    QTextStream&	istream,
    QPoint&		point
    )
{
    istream >> point.rx() >> point.ry();
    
    return istream;
}


QTextStream& operator <<(
    QTextStream&	ostream,
    const QRect&	rect
    )
{
    ostream << rect.left() << ' ' << rect.top() << ' ' <<
    rect.width() << ' ' << rect.height();
    
    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QRect&		rect
    )
{
    int left, top, width, height;
    
    istream >> left;
    istream >> top;
    istream >> width;
    istream >> height;
    
    rect.setRect( left, top, width, height );
    
    return istream;
}


// write palette to text stream
QTextStream& operator <<(
    QTextStream&	ostream,
    const QPalette&	palette
    )
{
    ostream << 
	palette.normal() << ' ' <<
	palette.disabled() << ' ' <<
	palette.active();
    
    return ostream;
}


// read palette from stream
QTextStream& operator >>(
    QTextStream&	istream,
    QPalette&		palette
    )
{
    QColorGroup normal, disabled, active;
    
    istream >> normal >> disabled >> active;
    
    palette.setNormal( normal );
    palette.setDisabled( disabled );
    palette.setActive( active );

    return istream;
}


QTextStream& operator <<(
    QTextStream&	ostream,
    const QColorGroup&	group
    )
{
    ostream << 
	group.foreground().rgb() << ' ' <<
	group.button().rgb() << ' ' <<
	group.light().rgb() << ' ' <<
	group.dark().rgb() << ' ' <<
	group.mid().rgb() << ' ' <<
	group.text().rgb() << ' ' <<
	group.base().rgb() << ' ' <<
	group.background().rgb() << ' ' <<
	group.midlight().rgb() << ' ' <<
	group.brightText().rgb() << ' ' <<
	group.buttonText().rgb() << ' ' <<
	group.shadow().rgb() << ' ' <<
	group.highlight().rgb() << ' ' <<
	group.highlightedText().rgb();
    
    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QColorGroup&	group
    )
{
    QRgb fore, button, light, dark, mid, text, base, background, 
        midlight, brightText, buttonText, shadow, highlight, highlightedText;
    
    istream >> fore >> button >> light >> dark >> mid >> text >> base 
            >> background >> midlight >> brightText >> buttonText 
            >> shadow >> highlight >> highlightedText;
    
    group.setColor( QColorGroup::Foreground, QColor( fore ) );
    group.setColor( QColorGroup::Button, QColor( button ) );
    group.setColor( QColorGroup::Light, QColor( light ) );
    group.setColor( QColorGroup::Midlight, QColor( midlight ) );
    group.setColor( QColorGroup::Dark, QColor( dark ) );
    group.setColor( QColorGroup::Mid, QColor( mid ) );
    group.setColor( QColorGroup::Text, QColor( text ) );
    group.setColor( QColorGroup::BrightText, QColor( brightText ) );
    group.setColor( QColorGroup::ButtonText, QColor( buttonText ) );
    group.setColor( QColorGroup::Base, QColor( base ) );
    group.setColor( QColorGroup::Background, QColor( background ) );
    group.setColor( QColorGroup::Shadow, QColor( shadow ) );
    group.setColor( QColorGroup::Highlight, QColor( highlight ) );
    group.setColor( QColorGroup::HighlightedText, QColor( highlightedText ) );
    
    return istream;
}
  

QTextStream& operator <<(
    QTextStream&	ostream,
    const QFont&	font
    )
{
    ostream << '\"' << font.family() << "\" " <<
	font.pointSize() << ' ' <<
	font.weight() << ' ' <<
	(int)font.italic() << ' ' <<
	(int)font.underline() << ' ' <<
	(int)font.strikeOut() << ' ' <<
	(int)font.fixedPitch() << ' ' <<
	(int)font.styleHint() << ' ' <<
	(int)font.charSet();

    return ostream;
}


QTextStream& operator >>(
    QTextStream&	istream,
    QFont&		font
    )
{
    QString family, str;
    
    // since family name might have spaces in it, loop until you get the 
    // closing quote which encloses the name
    do
    {
	istream >> str;
	family += str;
    } while( str[((int)str.length()) - 1] != '\"' );
    
    // remove leading and trailing "'s
    family = family.mid( 1, family.length()-2 );
    
    font.setFamily( family );

    int n;
    
    istream >> n;
    font.setPointSize( n );
    
    istream >> n;
    font.setWeight( n );
    
    istream >> n;
    font.setItalic( n );
    
    istream >> n;
    font.setUnderline( n );

    istream >> n;
    font.setStrikeOut( n );
    
    istream >> n;
    font.setFixedPitch( n );
    
    istream >> n;
    font.setStyleHint( (QFont::StyleHint)n );
    
    istream >> n;
    font.setCharSet( (QFont::CharSet)n );

    return istream;
}


void SaveBool(
    QTextStream&	ostream,
    const bool&	b
    )
{
    ostream << (b ? "TRUE" : "FALSE");
}


void RestoreBool(
    QTextStream&	istream,
    bool&		b
    )
{
    QString aStr;

    istream >> aStr;
    if( aStr == "TRUE" )
	b = TRUE;
    else
	b = FALSE;
}


DlgWidget * DlgWidget::GetParentForCodeGeneration(
    const QList<DlgWidget>& zOrdering
    )
{
    DlgWidget                *widget;
    QListIterator<DlgWidget> iter(zOrdering);

    //
    // A widget's parnt will be before it within the zOrder, and be capable of
    // having children, and will have a region that contains this widget.
    //
    while( (widget=iter()) )
        if(widget==this)
            break;
        else
            if ((widget->GetType()==DW_ButtonGroup ||
                 widget->GetType()==DW_GroupBox) &&
                 widget->GetWidget()->x()<=itsWidget->x() &&
                 widget->GetWidget()->y()<=itsWidget->y() &&
                 (widget->GetWidget()->x()+widget->GetWidget()->width())>=(itsWidget->x()+itsWidget->width())&&
                 (widget->GetWidget()->y()+widget->GetWidget()->height())>=(itsWidget->y()+itsWidget->height()))
                itsParentForSource=widget;

    return itsParentForSource;
}
